##############################################################################################
#
#  Worker Threads loosely based on QRunnable by Martin Fitzpatrick
#  https://www.pythonguis.com/tutorials/multithreading-pyside6-applications-qthreadpool/
#  License: MIT
#
##############################################################################################

import sys
import traceback
from dataclasses import dataclass
from typing import Callable

from PySide6.QtCore import QRunnable, Slot, Signal, QObject


@dataclass(frozen=True, slots=True)
class WorkerError:
    exctype: type
    value: Exception
    traceback: str
    args: tuple | None = None
    kwargs: dict | None = None

    def __str__(self):
        return f"{self.exctype}: {self.traceback}\n{self.value}"


class WorkerSignals(QObject):
    """
    Defines the signals available from a running worker thread.

    Supported signals are:

    finished
        The initial parameters passed to the worker when it was started, args and kwargs.

    error
        an instance of WorkerError.

    result
        object data returned from processing, if anything.

    progress
        anything to indicate the current state.

    """

    finished = Signal(tuple)
    error = Signal(WorkerError)
    result = Signal(object)
    progress = Signal(object)


class Worker(QRunnable):
    """
    Worker thread

    Inherits from QRunnable to handler worker thread setup, signals and wrap-up.

    DO NO. I REPEAT. DO NOT PUT @Slot() DECORATORS ON THE FUNCTIONS THAT YOU
    CONNECT TO THE SIGNALS. YOU WILL ONLY BE REWARDED WITH SEGFAULTS.

    :param callback: The function callback to run on this worker thread. Supplied args and
                     kwargs will be passed through to the runner.
    :param args: Arguments to pass to the callback function.
    :param no_progress_callback: If present (true or false), don't send progress signals.
    :param kwargs: Keywords to pass to the callback function.
    """

    def __init__(self, fn: Callable, *args, **kwargs):
        QRunnable.__init__(self)

        # Store constructor arguments (re-used for processing).
        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.signals = WorkerSignals()

        # Add the callback to our kwargs.
        if "no_progress_callback" not in kwargs:
            self.kwargs["progress_callback"] = self.signals.progress
        else:
            # Remove the no_progress_callback from kwargs.
            del self.kwargs["no_progress_callback"]

    @Slot()
    def run(self):
        """
        Initialise the runner function with passed args, kwargs.
        """

        # Retrieve args/kwargs here; and fire processing using them.
        try:
            result = self.fn(*self.args, **self.kwargs)
        except:
            traceback.print_exc()
            exctype, value = sys.exc_info()[:2]
            self.signals.error.emit(
                WorkerError(exctype, value, traceback.format_exc(), self.args, self.kwargs)
            )
        else:
            self.signals.result.emit(result)  # Return the result of the processing
        finally:
            self.signals.finished.emit((self.args, self.kwargs))  # Done



"""
This file contains a slightly modified version of the original function,
and serves as the only link to the comic_text_detector project:
https://github.com/dmMaze/comic-text-detector
(License: GNU General Public License v3.0)

Should a newer version of the comic_text_detector project be released,
replacing the files in the pcleaner/comic_text_detector folder with the
newer version should be sufficient to update the code, provided that the
coupling between this function and the rest of the code is not broken.

The code present in the pcleaner/comic_text_detector folder is a
slimmed-down version of the original project, containing only the
necessary files to run the model and the function below.
"""

import json
import uuid
from pathlib import Path
import multiprocessing as mp

from tqdm import tqdm
import torch
import numpy as np
import cv2
from logzero import logger

import pcleaner.config as cfg
from .comic_text_detector.inference import TextDetector
from .comic_text_detector.utils.io_utils import imwrite, NumpyEncoder
from .comic_text_detector.utils.textmask import REFINEMASK_ANNOTATION


def model2annotations(
    config_general: cfg.GeneralConfig,
    config_detector: cfg.TextDetectorConfig,
    model_path: Path,
    img_list: list[Path],
    save_dir: Path,
):
    """
    Run the model on a directory of images and produce the following
    for each image inside the save_dir directory:
    - A copy of the original image as a .png file.
    - A .png file containing the text mask, filename: <image_name>_mask.png.
    - A .json file containing each box of text, as well as other metadata,
      filename: <image_name>.json.

    For this modified version, include the image name and mask name in the
    json file.

    :param config_general: General configuration, part of the profile.
    :param config_detector: Text detector configuration, part of the profile.
    :param model_path: Path to the model file. This ends either in .pt or .onnx (torch or cv2 format).
    :param img_list: Path or a list of paths to an image or directory of images.
    :param save_dir: Path to the directory where the results will be saved.
    :return:
    """

    device = "cuda" if model_path.suffix == ".pt" else "cpu"
    print(f"Using device for text detection model: {device}")
    # Determine the number of processes to use
    num_processes = min(config_detector.concurrent_models, len(img_list))
    print(f"Using {num_processes} processes for text detection.")

    if num_processes > 1:

        mp.freeze_support()
        mp.set_start_method("spawn")

        with mp.Pool(num_processes) as pool:

            # Distribute the images evenly among the processes.
            batches = [list() for _ in range(num_processes)]
            for i, img_path in enumerate(img_list):
                batches[i % num_processes].append(img_path)

            args = [
                (batch, model_path, device, save_dir, config_general.input_size_scale)
                for batch in batches
            ]

            for _ in tqdm(pool.imap_unordered(process_image_batch, args), total=len(args)):
                pass  # do nothing, just iterate through the results

    else:
        model = TextDetector(model_path=str(model_path), input_size=1024, device=device)

        for index, img_path in enumerate(tqdm(img_list)):
            process_image(img_path, model, save_dir, config_general.input_size_scale)


def process_image_batch(args):
    img_batch, model_path, device, save_dir, image_scale = args
    model = TextDetector(model_path=str(model_path), input_size=1024, device=device)
    for img_path in img_batch:
        process_image(img_path, model, save_dir, image_scale)

    del model

    if device == "cuda":
        # Release CUDA resources.
        torch.cuda.ipc_collect()
        torch.cuda.empty_cache()


def process_image(img_path: Path, model: TextDetector, save_dir: Path, image_scale: float):
    """
    Process a single image using the TextDetector model.
    This generates a mask and a json file containing the text boxes.
    These both belong in the cache directory.

    :param img_path: The path to the image to process.
    :param model: The TextDetector model.
    :param save_dir: The directory where the results will be saved.
    :param image_scale: The scale to use when resizing the image (float > 0).
    """

    img = read_image(img_path, image_scale)

    # Prepend an index to prevent name clobbering between different files.
    prefix = f"{uuid.uuid4()}_"

    img_name = prefix + img_path.stem
    maskname = img_name + "_mask.png"

    # Make names absolute paths.
    img_name = str((save_dir / (img_name + ".png")).absolute())
    maskname = str((save_dir / maskname).absolute())

    mask, mask_refined, blk_list = model(
        img, refine_mode=REFINEMASK_ANNOTATION, keep_undetected_mask=True
    )
    blk_xyxy = []
    blk_dict_list = []
    for blk in blk_list:
        blk_xyxy.append(blk.xyxy)
        blk_dict_list.append(blk.to_dict())

    # Inject the img_name.png and mask name and original path into the json.
    data = {
        "image_path": img_name,
        "mask_path": maskname,
        "original_path": str(img_path),
        "scale": image_scale,
        "blk_list": blk_dict_list,
    }
    # Remove the suffix and add _raw.json
    json_path = Path(img_name).with_suffix(".json")
    json_path = json_path.with_stem(json_path.stem + "#raw")
    logger.debug(f"Saving json file to {json_path}")
    with open(json_path, "w", encoding="utf8") as f:
        json.dump(data, f, ensure_ascii=False, cls=NumpyEncoder, indent=4)
    imwrite(img_name, img)
    imwrite(maskname, mask_refined)


def read_image(path: Path | str, scale=1.0) -> np.ndarray:
    """
    Read image from path, scaling it if necessary.
    Then return an array of the image.

    :param path: Image path
    :param scale: Scale factor
    :return: Image array
    """

    img = cv2.imdecode(np.fromfile(str(path), dtype=np.uint8), cv2.IMREAD_COLOR)

    if scale != 1.0:
        height, width, channels = img.shape
        new_width = int(width * scale)
        new_height = int(height * scale)
        img = cv2.resize(img, (new_width, new_height))

    return img



from enum import Enum, auto

import PySide6.QtWidgets as Qw
from PySide6.QtCore import Slot, Signal, QEvent

from typing import Callable, Any
from logzero import logger


class HookState(Enum):
    Start = auto()
    CheckHook = auto()
    End = auto()


# noinspection PyPep8Naming
class CComboBox(Qw.QComboBox):
    """
    Extends the functionality with custom helpers
    And includes a secondary array for data linked to each item
    """

    _pre_change_hook: Callable[[], bool] | None = None
    _prev_index: int = -1
    _next_index: int = -1
    _hook_state: HookState = HookState.Start
    hookedCurrentIndexChanged: Signal = Signal(int)

    def __init__(self, parent=None):
        Qw.QComboBox.__init__(self, parent)
        self._linked_data = []
        # self.installEventFilter(self)
        self.currentIndexChanged.connect(self.forward_current_index_changed_hooked)

    def set_pre_change_hook(self, hook: Callable[[], bool]):
        self._pre_change_hook = hook

    def emit_hooked_change(self, index: int):
        """
        Emit the hooked change signal, but only if the index has changed.
        Remember the previous index.
        :param index: The new index.
        """
        if self._prev_index != index:
            self._prev_index = index
        self.hookedCurrentIndexChanged.emit(index)

    @Slot(int)
    def forward_current_index_changed_hooked(self, new_index: int):
        """
        Forward the current index changed signal, but only if the hook returns True.

        :param new_index: The new index
        """
        if self._pre_change_hook is None:
            self.hookedCurrentIndexChanged.emit(new_index)
            if self._prev_index != new_index:
                self._prev_index = new_index
            return

        match self._hook_state:
            case HookState.Start:
                # Go back to the previous index temporarily, to check the hook.
                self._next_index = new_index
                self._hook_state = HookState.CheckHook
                self.setCurrentIndex(
                    self._prev_index if self._prev_index != -1 else self.currentIndex()
                )
            case HookState.CheckHook:
                if self._pre_change_hook():
                    self._hook_state = HookState.End
                    self.setCurrentIndex(self._next_index)
                    self.emit_hooked_change(self._next_index)
                else:
                    self._hook_state = HookState.Start
            case HookState.End:
                self._hook_state = HookState.Start

    def clear(self):
        Qw.QComboBox.clear(self)
        self._linked_data.clear()

    def addTextItemLinkedData(self, text: str, data: any):
        self.addItem(text)
        self._linked_data.append(data)

    def setCurrentIndexByText(self, text: str):
        self.setCurrentIndex(self.findText(text))

    def setCurrentIndexByLinkedData(self, data: any):
        self.setCurrentIndex(self._linked_data.index(data))

    def indexLinkedData(self, data: any):
        return self._linked_data.index(data)

    @Slot(int)
    def setCurrentIndex(self, index: int) -> None:
        """
        Ensure the index is valid.
        :param index:
        """
        Qw.QComboBox.setCurrentIndex(self, min(max(index, 0), self.count() - 1))

    def currentLinkedData(self):
        try:
            return self._linked_data[self.currentIndex()]
        except IndexError:
            logger.error("No linked data found for current index")
            logger.error("Current index:", self.currentIndex())
            logger.error("Linked data:", self._linked_data)



import platform
import subprocess
from math import ceil
from pathlib import Path
import difflib

from logzero import logger


def f_plural(value, singular: str, plural: str):
    """
    Selects which form to use based on the value.
    """
    return singular if value == 1 else plural


def scale_length_rounded(size: int, scale: float) -> int:
    """
    Scale the given size by the given scale.
    Round up to prevent nulling small values.

    :param size: Size to scale.
    :param scale: Scale factor.
    :return: Scaled size.
    """
    return ceil(size * scale)


def scale_area_rounded(area: int, scale: float) -> int:
    """
    Scale the given area by the given scale.
    Round up to prevent nulling small values.

    :param area: Area to scale.
    :param scale: Scale factor.
    :return: Scaled area.
    """
    return ceil(area * scale * scale)


def open_file(path: Path):
    """
    Open any given file with the default application.
    """
    logger.info(f"Opening file {path}")
    try:
        if platform.system() == "Linux":
            subprocess.run(["xdg-open", path])
        elif platform.system() == "Windows":
            subprocess.run(["start", path])
        elif platform.system() == "Darwin":
            subprocess.run(["open", path])
    except Exception as e:
        logger.exception(e)


def closest_match(word: str, choices: list[str]) -> str | None:
    """
    Return the closest match for the given word in the list of choices.
    If no good match is found, return None.
    """
    if word in choices:
        return word
    else:
        # Find the closest match using difflib:
        closest = difflib.get_close_matches(word, choices, 1, 0.5)  # 0.6 is the default threshold
        if closest:
            return str(closest[0])
        else:
            return None



import PySide6.QtWidgets as Qw
import PySide6.QtCore as Qc
import PySide6.QtGui as Qg

"""
Extend a PushButton to display a color, and allow the user to change it
with a color dialog.

The color is stored as a QColor object.
Optionally, use an alpha channel.

The color can be set or retrieved as a 3 or 4 tuple of ints.


The button has a custom painter that draws a rectangle with the color,
and if the color is None, it defaults to white.
If the color has an alpha channel, the rectangle is drawn with a checkerboard]
pattern behind it.
"""


# Follow Qt's naming convention of CamelCase for functions, signals, and classes
# noinspection PyPep8Naming
class ColorButton(Qw.QPushButton):

    colorChanged = Qc.Signal(Qg.QColor)

    def __init__(
        self,
        parent=None,
        color: tuple[int, int, int, int] | tuple[int, int, int] = (255, 255, 255, 255),
        dialog_title: str = "Change Color",
    ):
        super().__init__(parent)
        self._color: Qg.QColor = Qg.QColor(*color)
        self.use_alpha = len(color) == 4
        self.dialog_title = dialog_title
        self.setFixedHeight(32)
        self.clicked.connect(self._open_color_dialog)

    def paintEvent(self, event: Qg.QPaintEvent) -> None:
        painter = Qg.QPainter(self)
        painter.setRenderHint(Qg.QPainter.Antialiasing)
        self._draw_color(painter)

    def _open_color_dialog(self) -> None:
        color = Qw.QColorDialog.getColor(
            self._color,
            self,
            self.dialog_title,
            Qw.QColorDialog.ShowAlphaChannel if self.use_alpha else 0,
        )
        if color.isValid() and color != self._color:
            self._color = color
            self.repaint()
            self.colorChanged.emit(color)

    def _draw_color(self, painter: Qg.QPainter) -> None:
        if self._color.alpha() != 255:
            self._draw_checkerboard(painter)
        painter.fillRect(0, 0, self.width(), self.height(), self._color)

    def _draw_checkerboard(self, painter: Qg.QPainter) -> None:
        for i in range(0, self.width(), 16):
            for j in range(0, self.height(), 16):
                if (i + j) % 32 == 0:
                    painter.fillRect(i, j, 16, 16, Qg.Qt.lightGray)
                else:
                    painter.fillRect(i, j, 16, 16, Qg.Qt.darkGray)

    def color(self) -> tuple[int, int, int, int]:
        return self._color.red(), self._color.green(), self._color.blue(), self._color.alpha()

    def setColor(self, color: tuple[int, int, int, int]) -> None:
        self._color = Qg.QColor(*color)
        self.repaint()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ImageDetails.ui'
##
## Created by: Qt User Interface Compiler version 6.4.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QHBoxLayout, QLabel,
    QListView, QListWidget, QListWidgetItem, QPushButton,
    QSizePolicy, QSpacerItem, QVBoxLayout, QWidget)

class Ui_ImageDetails(object):
    def setupUi(self, ImageDetails):
        if not ImageDetails.objectName():
            ImageDetails.setObjectName(u"ImageDetails")
        ImageDetails.resize(800, 600)
        self.verticalLayout = QVBoxLayout(ImageDetails)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.listWidget = QListWidget(ImageDetails)
        self.listWidget.setObjectName(u"listWidget")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidget.sizePolicy().hasHeightForWidth())
        self.listWidget.setSizePolicy(sizePolicy)
        self.listWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.listWidget.setProperty("showDropIndicator", False)
        self.listWidget.setFlow(QListView.LeftToRight)
        self.listWidget.setLayoutMode(QListView.SinglePass)
        self.listWidget.setViewMode(QListView.IconMode)
        self.listWidget.setUniformItemSizes(True)
        self.listWidget.setItemAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignVCenter)

        self.verticalLayout.addWidget(self.listWidget)

        self.widget_image = QWidget(ImageDetails)
        self.widget_image.setObjectName(u"widget_image")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.widget_image.sizePolicy().hasHeightForWidth())
        self.widget_image.setSizePolicy(sizePolicy1)

        self.verticalLayout.addWidget(self.widget_image)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label_step = QLabel(ImageDetails)
        self.label_step.setObjectName(u"label_step")

        self.horizontalLayout.addWidget(self.label_step)

        self.horizontalSpacer_2 = QSpacerItem(20, 0, QSizePolicy.Maximum, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.label = QLabel(ImageDetails)
        self.label.setObjectName(u"label")

        self.horizontalLayout.addWidget(self.label)

        self.label_size = QLabel(ImageDetails)
        self.label_size.setObjectName(u"label_size")

        self.horizontalLayout.addWidget(self.label_size)

        self.horizontalSpacer_3 = QSpacerItem(20, 0, QSizePolicy.Maximum, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_3)

        self.label_fil_name = QLabel(ImageDetails)
        self.label_fil_name.setObjectName(u"label_fil_name")

        self.horizontalLayout.addWidget(self.label_fil_name)

        self.horizontalSpacer = QSpacerItem(40, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.pushButton_zoom_in = QPushButton(ImageDetails)
        self.pushButton_zoom_in.setObjectName(u"pushButton_zoom_in")
        icon = QIcon(QIcon.fromTheme(u"zoom-in"))
        self.pushButton_zoom_in.setIcon(icon)
        self.pushButton_zoom_in.setFlat(True)

        self.horizontalLayout.addWidget(self.pushButton_zoom_in)

        self.pushButton_zoom_out = QPushButton(ImageDetails)
        self.pushButton_zoom_out.setObjectName(u"pushButton_zoom_out")
        icon1 = QIcon(QIcon.fromTheme(u"zoom-out"))
        self.pushButton_zoom_out.setIcon(icon1)
        self.pushButton_zoom_out.setFlat(True)

        self.horizontalLayout.addWidget(self.pushButton_zoom_out)

        self.pushButton_zoom_reset = QPushButton(ImageDetails)
        self.pushButton_zoom_reset.setObjectName(u"pushButton_zoom_reset")
        icon2 = QIcon(QIcon.fromTheme(u"zoom-original"))
        self.pushButton_zoom_reset.setIcon(icon2)
        self.pushButton_zoom_reset.setFlat(True)

        self.horizontalLayout.addWidget(self.pushButton_zoom_reset)


        self.verticalLayout.addLayout(self.horizontalLayout)


        self.retranslateUi(ImageDetails)

        QMetaObject.connectSlotsByName(ImageDetails)
    # setupUi

    def retranslateUi(self, ImageDetails):
        ImageDetails.setWindowTitle(QCoreApplication.translate("ImageDetails", u"Form", None))
        self.label_step.setText(QCoreApplication.translate("ImageDetails", u"<Current Step>", None))
        self.label.setText(QCoreApplication.translate("ImageDetails", u"Size:", None))
        self.label_size.setText(QCoreApplication.translate("ImageDetails", u"TextLabel", None))
        self.label_fil_name.setText(QCoreApplication.translate("ImageDetails", u"<File name>", None))
#if QT_CONFIG(tooltip)
        self.pushButton_zoom_in.setToolTip(QCoreApplication.translate("ImageDetails", u"Zoom in", None))
#endif // QT_CONFIG(tooltip)
        self.pushButton_zoom_in.setText("")
#if QT_CONFIG(tooltip)
        self.pushButton_zoom_out.setToolTip(QCoreApplication.translate("ImageDetails", u"Zoom out", None))
#endif // QT_CONFIG(tooltip)
        self.pushButton_zoom_out.setText("")
#if QT_CONFIG(tooltip)
        self.pushButton_zoom_reset.setToolTip(QCoreApplication.translate("ImageDetails", u"Reset zoom", None))
#endif // QT_CONFIG(tooltip)
        self.pushButton_zoom_reset.setText("")
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'NewProfile.ui'
##
## Created by: Qt User Interface Compiler version 6.4.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QApplication, QDialog, QDialogButtonBox,
    QFormLayout, QHBoxLayout, QLabel, QLineEdit,
    QPushButton, QRadioButton, QSizePolicy, QSpacerItem,
    QVBoxLayout, QWidget)

class Ui_NewProfile(object):
    def setupUi(self, NewProfile):
        if not NewProfile.objectName():
            NewProfile.setObjectName(u"NewProfile")
        NewProfile.resize(556, 234)
        self.verticalLayout = QVBoxLayout(NewProfile)
        self.verticalLayout.setSpacing(12)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.label_default_protection_hint = QLabel(NewProfile)
        self.label_default_protection_hint.setObjectName(u"label_default_protection_hint")
        self.label_default_protection_hint.setWordWrap(True)

        self.verticalLayout.addWidget(self.label_default_protection_hint)

        self.formLayout = QFormLayout()
        self.formLayout.setObjectName(u"formLayout")
        self.radioButton_default_location = QRadioButton(NewProfile)
        self.radioButton_default_location.setObjectName(u"radioButton_default_location")
        self.radioButton_default_location.setChecked(True)

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.radioButton_default_location)

        self.radioButton_custom_location = QRadioButton(NewProfile)
        self.radioButton_custom_location.setObjectName(u"radioButton_custom_location")

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.radioButton_custom_location)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.lineEdit_location = QLineEdit(NewProfile)
        self.lineEdit_location.setObjectName(u"lineEdit_location")
        self.lineEdit_location.setEnabled(False)

        self.horizontalLayout.addWidget(self.lineEdit_location)

        self.pushButton_browse_location = QPushButton(NewProfile)
        self.pushButton_browse_location.setObjectName(u"pushButton_browse_location")
        self.pushButton_browse_location.setEnabled(False)
        icon = QIcon()
        iconThemeName = u"folder-open"
        if QIcon.hasThemeIcon(iconThemeName):
            icon = QIcon.fromTheme(iconThemeName)
        else:
            icon.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_browse_location.setIcon(icon)

        self.horizontalLayout.addWidget(self.pushButton_browse_location)


        self.formLayout.setLayout(2, QFormLayout.FieldRole, self.horizontalLayout)

        self.label = QLabel(NewProfile)
        self.label.setObjectName(u"label")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label)

        self.lineEdit_name = QLineEdit(NewProfile)
        self.lineEdit_name.setObjectName(u"lineEdit_name")

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.lineEdit_name)

        self.label_default_path = QLabel(NewProfile)
        self.label_default_path.setObjectName(u"label_default_path")
        self.label_default_path.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByKeyboard|Qt.TextSelectableByMouse)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.label_default_path)


        self.verticalLayout.addLayout(self.formLayout)

        self.verticalSpacer = QSpacerItem(20, 5, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(2)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.label_warning_icon = QLabel(NewProfile)
        self.label_warning_icon.setObjectName(u"label_warning_icon")
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_warning_icon.sizePolicy().hasHeightForWidth())
        self.label_warning_icon.setSizePolicy(sizePolicy)
        self.label_warning_icon.setText(u"")

        self.horizontalLayout_2.addWidget(self.label_warning_icon)

        self.label_warning_message = QLabel(NewProfile)
        self.label_warning_message.setObjectName(u"label_warning_message")
        font = QFont()
        font.setItalic(True)
        self.label_warning_message.setFont(font)

        self.horizontalLayout_2.addWidget(self.label_warning_message)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.buttonBox = QDialogButtonBox(NewProfile)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Save)

        self.verticalLayout.addWidget(self.buttonBox)

        QWidget.setTabOrder(self.lineEdit_name, self.radioButton_default_location)
        QWidget.setTabOrder(self.radioButton_default_location, self.radioButton_custom_location)
        QWidget.setTabOrder(self.radioButton_custom_location, self.lineEdit_location)
        QWidget.setTabOrder(self.lineEdit_location, self.pushButton_browse_location)

        self.retranslateUi(NewProfile)
        self.buttonBox.accepted.connect(NewProfile.accept)
        self.buttonBox.rejected.connect(NewProfile.reject)
        self.radioButton_custom_location.toggled.connect(self.lineEdit_location.setEnabled)
        self.radioButton_custom_location.toggled.connect(self.pushButton_browse_location.setEnabled)

        QMetaObject.connectSlotsByName(NewProfile)
    # setupUi

    def retranslateUi(self, NewProfile):
        NewProfile.setWindowTitle(QCoreApplication.translate("NewProfile", u"New Profile", None))
        self.label_default_protection_hint.setText(QCoreApplication.translate("NewProfile", u"The default profile cannot be overwritten. Save a new profile with your changes:", None))
        self.radioButton_default_location.setText(QCoreApplication.translate("NewProfile", u"Use the default location", None))
        self.radioButton_custom_location.setText(QCoreApplication.translate("NewProfile", u"Use a different location:", None))
        self.pushButton_browse_location.setText("")
        self.label.setText(QCoreApplication.translate("NewProfile", u"Profile Name:", None))
        self.label_default_path.setText(QCoreApplication.translate("NewProfile", u"<default path>", None))
        self.label_warning_message.setText(QCoreApplication.translate("NewProfile", u"<some warning>", None))
    # retranslateUi




import shutil
from functools import partial
from pathlib import Path
from importlib import resources

import PySide6.QtCore as Qc
import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw
from PySide6.QtCore import Signal, Slot
from logzero import logger

import pcleaner.cli_utils as cu
import pcleaner.helpers as hp
import pcleaner.config as cfg
import pcleaner.gui.profile_parser as pp
import pcleaner.gui.gui_utils as gu
import pcleaner.profile_cli as pc
from pcleaner.gui.ui_generated_files.ui_Mainwindow import Ui_MainWindow
from pcleaner.gui.file_table import Column
from pcleaner import __display_name__, __version__
from pcleaner import data
import pcleaner.gui.new_profile_driver as npd


# noinspection PyUnresolvedReferences
class MainWindow(Qw.QMainWindow, Ui_MainWindow):

    config: cfg.Config = None
    # glossary: st.Glossary = None
    processing: bool = False

    # text_params_changed = Signal(st.Glossary)
    # text_output_changed = Signal()
    # abort_translation_worker = Signal()

    label_stats: Qw.QLabel

    toolBox_profile: pp.ProfileToolBox

    def __init__(self):
        Qw.QMainWindow.__init__(self)
        self.setupUi(self)
        self.setWindowTitle(f"{__display_name__} {__version__}")
        self.setWindowIcon(Qg.QIcon(":/logo-tiny.png"))
        self.processing = False

        self.config = cfg.load_config()

        self.initialize_ui()
        #
        # self.threadpool = Qc.QThreadPool.globalInstance()
        # logger.info(f"Multithreading with maximum {self.threadpool.maxThreadCount()} threads")
        #
        # self.config = cfg.Config.load()
        # # Set debug flag.
        # self.config.tl_mock = mock
        #
        # logger.debug(f"Loaded config: {self.config.safe_dump()}")
        # self.load_config_to_ui()
        # # Share config with the file table.
        # self.file_table.set_config(self.config)
        #
        # self.load_glossary()

    def initialize_ui(self):
        self.hide_progress()
        self.start_button_enabled(False)
        self.show_button_start()
        # self.update_input_buttons()
        self.set_up_statusbar()
        self.initialize_profiles()
        self.initialize_analytics_view()

        # Allow the table to accept file drops and hide the ID column.
        self.file_table.setAcceptDrops(True)
        self.file_table.setColumnHidden(Column.ID, True)
        self.file_table.setColumnWidth(Column.FILENAME, 200)
        self.file_table.setColumnWidth(Column.STATUS, 200)

        # Connect signals.
        self.connect_combobox_slots()
        self.comboBox_current_profile.hookedCurrentIndexChanged.connect(self.change_current_profile)

        return

        self.checkBox_file_fixed_dir.toggled.connect(self.fixed_output_dir_toggled)
        self.checkBox_use_glossary.toggled.connect(self.use_glossary_toggled)
        self.checkBox_extra_quote_protection.toggled.connect(self.extra_quote_protection_toggled)
        self.pushButton_file_dir_browse.clicked.connect(self.browse_file_out_dir)
        self.lineEdit_file_out_dir.textEdited.connect(partial(self.fixed_file_out_updated, False))
        self.lineEdit_file_out_dir.editingFinished.connect(
            partial(self.fixed_file_out_updated, True)
        )

        self.pushButton_glossary_file_browse.clicked.connect(self.browse_glossary_file)
        self.lineEdit_glossary_file.textEdited.connect(
            partial(self.glossary_file_updated, self, False)
        )
        self.lineEdit_glossary_file.editingFinished.connect(
            partial(self.glossary_file_updated, self, True)
        )
        self.pushButton_glossary_help.clicked.connect(self.show_glossary_help)

        self.pushButton_api_config.clicked.connect(self.configure_api)
        self.pushButton_refresh.clicked.connect(self.load_config_to_ui)

        self.text_output_changed.connect(self.file_table.update_all_output_filenames)
        self.text_params_changed.connect(self.file_table.update_all_text_params)

        self.file_table.itemSelectionChanged.connect(self.update_input_buttons)
        self.pushButton_file_add.clicked.connect(self.file_table.browse_add_file)
        self.pushButton_file_preview.clicked.connect(self.file_table.preview_selected_file)
        self.pushButton_file_remove.clicked.connect(self.file_table.remove_selected_file)
        self.pushButton_file_remove_all.clicked.connect(self.file_table.remove_all_files)

        self.pushButton_start.clicked.connect(self.start_translating)
        self.pushButton_abort.clicked.connect(self.abort_translating)

        self.file_table.request_text_param_update.connect(self.update_file_table_params)
        self.file_table.ready_for_translation.connect(self.ready_to_translate)
        self.file_table.not_ready_for_translation.connect(self.not_ready_to_translate)
        self.file_table.statusbar_message.connect(self.statusbar.showMessage)
        self.file_table.recalculate_char_total.connect(self.recalculate_char_total)

    def closeEvent(self, event: Qg.QCloseEvent):
        """
        Notify config on close.
        """
        logger.info("Closing window.")
        # self.abort_translation_worker.emit()
        # if self.threadpool.activeThreadCount():
        #     self.statusbar.showMessage("Waiting for threads to finish...")
        #     # Process Qt events so that the message shows up.
        #     Qc.QCoreApplication.processEvents()
        #     self.threadpool.waitForDone()
        #
        # nuke_epub_cache()
        # event.accept()

    def connect_combobox_slots(self):
        """
        Connect the combobox slots.
        """
        # self.comboBox_lang_from.currentIndexChanged.connect(self.lang_from_updated)
        # self.comboBox_lang_to.currentIndexChanged.connect(self.lang_to_updated)
        pass

    def disconnect_combobox_slots(self):
        """
        Disconnect the combobox slots. This prevents erroneous signals from being sent during
        setup of new language options.
        """
        # self.comboBox_lang_from.currentIndexChanged.disconnect()
        # self.comboBox_lang_to.currentIndexChanged.disconnect()
        pass

    # """
    # Config interactions
    # """
    #
    # def load_config_to_ui(self):
    #     """
    #     Apply data from config to ui widgets.
    #     """
    #     logger.debug("Loading config to UI.")
    #
    #     self.checkBox_file_fixed_dir.setChecked(self.config.use_fixed_output_path)
    #     self.lineEdit_file_out_dir.setText(self.config.fixed_output_path)
    #
    #     self.lineEdit_glossary_file.setText(self.config.glossary_path)
    #     self.checkBox_use_glossary.setChecked(self.config.use_glossary)
    #     self.checkBox_extra_quote_protection.setChecked(self.config.use_quote_protection)
    #
    #     self.fixed_output_dir_enabled(self.config.use_fixed_output_path)
    #     self.glossary_enabled(self.config.use_glossary)
    #
    #     # Ignore the mock because it cannot give language options. It is only to be used for translation.
    #     translator = self.open_translator(use_mock=False)
    #
    #     self.show_api_status(translator is not None)
    #     self.update_current_usage(translator)
    #
    #     if translator is not None:
    #         # Disconnect the slots to prevent signals from being sent during setup of new language options.
    #         self.disconnect_combobox_slots()  # vvv
    #         # Load language options from API.
    #         self.comboBox_lang_from.clear()
    #         self.comboBox_lang_to.clear()
    #
    #         # Add automatic option.
    #         self.comboBox_lang_from.addTextItemLinkedData("Detect", "")
    #
    #         for lang in translator.get_source_languages():
    #             self.comboBox_lang_from.addTextItemLinkedData(lang.name, lang.code)
    #
    #         for lang in translator.get_target_languages():
    #             self.comboBox_lang_to.addTextItemLinkedData(lang.name, lang.code)
    #
    #         self.connect_combobox_slots()  # ^^^
    #
    #         # Try to select the configured languages.
    #         try:
    #             self.comboBox_lang_from.setCurrentIndexByLinkedData(self.config.lang_from)
    #         except ValueError:
    #             logger.debug(f"Configured lang_from '{self.config.lang_from}' not found")
    #
    #         try:
    #             self.comboBox_lang_to.setCurrentIndexByLinkedData(self.config.lang_to)
    #         except ValueError:
    #             logger.debug(f"Configured lang_to '{self.config.lang_to}' not found")
    #     else:
    #         self.statusbar.showMessage("Error connecting to the API.")
    #
    # def lang_from_updated(self):
    #     """
    #     Copy lang_from from the combobox into the config and save it.
    #     """
    #     self.config.lang_from = self.comboBox_lang_from.currentLinkedData()
    #     logger.debug(f"Saving lang_from: {self.config.lang_from}")
    #     self.config.save()
    #
    # def lang_to_updated(self):
    #     """
    #     Copy lang_to from the combobox into the config and save it.
    #     """
    #     self.config.lang_to = self.comboBox_lang_to.currentLinkedData()
    #     logger.debug(f"Saving lang_to: {self.config.lang_to}")
    #     self.text_output_changed.emit()
    #     self.config.save()
    #
    # def fixed_output_dir_toggled(self):
    #     """
    #     Enable or disable the fixed output directory checkbox and save it to the config.
    #     """
    #     self.config.use_fixed_output_path = self.checkBox_file_fixed_dir.isChecked()
    #     self.fixed_output_dir_enabled(self.config.use_fixed_output_path)
    #     self.text_output_changed.emit()
    #     self.config.save()
    #
    # def browse_file_out_dir(self):
    #     """
    #     Browse for a directory to save files to.
    #     """
    #     dir_path = Qw.QFileDialog.getExistingDirectory(
    #         self, "Select output directory", self.config.fixed_output_path
    #     )
    #     if dir_path:
    #         self.lineEdit_file_out_dir.setText(dir_path)
    #         self.config.fixed_output_path = dir_path
    #         self.text_output_changed.emit()
    #         self.config.save()
    #
    # def fixed_file_out_updated(self, save: bool, *_):
    #     """
    #     Copy fixed_file_out from the lineedit into the config and save it.
    #     Don't save until the line edit signals the end of editing.
    #     Throw away the text in the line edit that the signal passes along by using *_.
    #     This is because the signal for editing finished does not pass along the text,
    #     making it unreliable.
    #
    #     :param save: Whether to save the config.
    #     """
    #     self.config.fixed_output_path = self.lineEdit_file_out_dir.text()
    #     if save:
    #         self.config.save()
    #     self.text_output_changed.emit()
    #
    # def use_glossary_toggled(self):
    #     """
    #     Enable or disable the glossary checkbox and save it to the config.
    #     """
    #     self.config.use_glossary = self.checkBox_use_glossary.isChecked()
    #     self.glossary_enabled(self.config.use_glossary)
    #     self.config.save()
    #     if self.config.use_glossary:
    #         self.load_glossary()
    #     self.text_params_changed.emit(self.glossary)
    #
    # def browse_glossary_file(self):
    #     """
    #     Browse for a glossary file.
    #     Accepts whatever pyexcel can handle.
    #     """
    #     file_path = Qw.QFileDialog.getOpenFileName(
    #         self, "Select glossary file", self.config.glossary_path
    #     )
    #     if file_path:
    #         self.lineEdit_glossary_file.setText(file_path[0])
    #         self.config.glossary_path = file_path[0]
    #         self.config.save()
    #         self.load_glossary()
    #
    # def glossary_file_updated(self, save: bool, *_):
    #     """
    #     Copy glossary_file from the lineedit into the config and save it.
    #     Don't save until the line edit signals the end of editing.
    #     Throw away the text in the line edit that the signal passes along by using *_.
    #     This is because the signal for editing finished does not pass along the text,
    #     making it unreliable.
    #
    #     :param save: Whether to save the config.
    #     """
    #     self.config.glossary_path = self.lineEdit_glossary_file.text()
    #     if save:
    #         self.config.save()
    #         self.load_glossary()
    #
    # def extra_quote_protection_toggled(self):
    #     """
    #     Enable or disable the extra quote protection checkbox and save it to the config.
    #     """
    #     self.config.use_quote_protection = self.checkBox_extra_quote_protection.isChecked()
    #     self.config.save()
    #     self.text_params_changed.emit(self.glossary)
    #
    # """
    # Dialogs
    # """
    #
    # def configure_api(self):
    #     """
    #     Configure the DeepL API.
    #     """
    #     api_conf_dialog = ConfigureAccount(self, self.config)
    #     response = api_conf_dialog.exec()
    #     if response == Qw.QDialog.Accepted:
    #         # Adopt changes from the dialog.
    #         self.config.api_key = api_conf_dialog.lineEdit_api_key.text()
    #         self.config.save()
    #         self.load_config_to_ui()
    #
    # """
    # Misc helpers
    # """
    #
    # # def open_translator(self, use_mock: bool = True) -> deepl.Translator | None:
    # #     """
    # #     Open a translator instance.
    # #
    # #     :return: translator instance and whether it was successful
    # #     """
    # #
    # #     if self.config.api_key == "":
    # #         # Fail silently if no API key is set, since this is the default value.
    # #         logger.warning("No API key set")
    # #         return None
    # #     try:
    # #         if self.config.tl_mock and use_mock:
    # #             logger.info("Opening mock translator.")
    # #             return deepl.Translator(
    # #                 auth_key="1234567890",
    # #                 server_url="http://localhost:3000",
    # #             )
    # #         else:
    # #             logger.info("Opening translator.")
    # #             translator = deepl.Translator(auth_key=self.config.api_key)
    # #             # Test the api, since initialization doesn't mean success.
    # #             translator.get_source_languages()
    # #             return translator
    # #     except Exception as e:
    # #         show_warning(self, "API Error", f"Failed connect to DeepL API\n\n{e}")
    # #         return None
    #
    # def update_file_table_params(self):
    #     """
    #     Update the file table with the current parameters.
    #     """
    #     self.text_params_changed.emit(self.glossary)
    #
    def initialize_analytics_view(self):
        """
        Set up the text edit for analytics.
        """
        # Set the font to monospace, using the included font.
        # The font is stored in the data module. Noto Mono is a free font.
        # Load it from file to be cross platform.
        font_path = resources.path(data, "NotoMono-Regular.ttf")
        font_id = Qg.QFontDatabase.addApplicationFont(str(font_path))
        if font_id != -1:
            logger.info("Loaded included font")
        else:
            logger.error(f"Failed to load included font. Using backup monospace font")
        self.textEdit_analytics.setReadOnly(True)
        self.textEdit_analytics.setLineWrapMode(Qw.QTextEdit.NoWrap)

    """
    Profiles
    """

    def initialize_profiles(self):
        """
        Load the available profiles and display the default profile.
        """
        all_profiles: list[tuple[str, Path | None]] = [(cfg.DEFAULT_PROFILE_NAME, None)]
        for profile_name, profile_path in self.config.saved_profiles.items():
            all_profiles.append((profile_name, profile_path))

        logger.debug(f"Found profiles: {all_profiles}")

        self.comboBox_current_profile.clear()
        for profile_name, profile_path in all_profiles:
            self.comboBox_current_profile.addTextItemLinkedData(profile_name, profile_path)

        # Set the current profile to the default profile.
        # If this is the default profile, then staying on the 0th index is fine.
        if self.config.default_profile:
            self.comboBox_current_profile.setCurrentIndexByText(self.config.default_profile)

        # Populate default profile list.
        self.menu_set_default_profile.clear()
        for profile_name, profile_path in all_profiles:
            action = Qg.QAction(profile_name, self)
            action.setCheckable(True)
            action.triggered.connect(partial(self.handle_set_default_profile, profile_name))
            self.menu_set_default_profile.addAction(action)
            action.setChecked(
                profile_name
                == (
                    self.config.default_profile
                    if self.config.default_profile
                    else cfg.DEFAULT_PROFILE_NAME
                )
            )

        # Load the ProfileToolBox widget.
        self.toolBox_profile = pp.ProfileToolBox(self)
        inner_layout = Qw.QVBoxLayout()
        inner_layout.setContentsMargins(0, 0, 0, 0)
        inner_layout.addWidget(self.toolBox_profile)
        self.toolBox_profile_frame.setLayout(inner_layout)
        self.config.load_profile(self.config.default_profile)

        # Load the structure.
        structure = pp.parse_profile_structure(self.config.current_profile)
        self.toolBox_profile.load_profile_structure(structure)

        self.load_current_profile()
        self.comboBox_current_profile.set_pre_change_hook(self.profile_change_check)

        # Connect signals.
        self.toolBox_profile.values_changed.connect(self.handle_profile_values_changed)
        self.pushButton_reset_profile.clicked.connect(self.reset_profile)
        self.pushButton_save_profile.clicked.connect(self.save_profile)
        self.action_save_profile.triggered.connect(self.save_profile)
        self.action_save_profile_as.triggered.connect(partial(self.save_profile, save_as=True))
        self.action_import_profile.triggered.connect(self.import_profile)
        self.action_new_profile.triggered.connect(partial(self.save_profile, make_new=True))
        self.action_delete_profile.triggered.connect(self.delete_profile)

    def handle_set_default_profile(self, profile_name: str):
        """
        Set the default profile in the config.
        """
        logger.debug(f"Setting default profile to {profile_name}")
        self.config.default_profile = (
            profile_name if profile_name != cfg.DEFAULT_PROFILE_NAME else None
        )
        self.config.save()
        # Update the menu.
        for action in self.menu_set_default_profile.actions():
            action.setChecked(action.text() == profile_name)

    def import_profile(self):
        """
        Open a file picker and add the profile to the profile list.
        """
        logger.debug("Importing profile.")
        file_path = Qw.QFileDialog.getOpenFileName(
            self,
            "Import Profile",
            "",
            "Profile Files (*.conf)",
        )[0]
        if file_path:
            logger.debug(f"Importing profile from {file_path}")
            profile_name = Path(file_path).stem
            success, msg = pc.add_profile(self.config, profile_name, file_path)
            if success:
                gu.show_info(self, "Profile Imported", msg)
            else:
                gu.show_warning(self, "Import Error", msg)
                return

            self.add_new_profile_to_gui(profile_name, file_path)

    def delete_profile(self):
        """
        Ask and then delete the current profile.
        Of course, the default profile cannot be deleted.
        :return:
        """
        # if not self.profile_change_check():
        #     return
        logger.debug("Deleting profile.")
        profile_name = self.comboBox_current_profile.currentText()

        if self.comboBox_current_profile.currentText() == cfg.DEFAULT_PROFILE_NAME:
            gu.show_warning(self, "Failed to Delete", "The default profile cannot be deleted.")
            return
        response = gu.show_question(
            self, "Delete Profile", f"Are you sure you want to delete the profile {profile_name}?"
        )
        if response == Qw.QMessageBox.Yes:
            profile_file = self.config.saved_profiles[profile_name]
            self.config.remove_profile(profile_name)
            self.config.save()
            if profile_file.is_file():
                try:
                    profile_file.unlink()
                except OSError as e:
                    gu.show_warning(
                        self, "Delete Error", "Failed to delete the profile\n" + e.what()
                    )

            self.toolBox_profile.reset_all()  # To suppress the change check.
            self.comboBox_current_profile.removeItem(self.comboBox_current_profile.currentIndex())
            for action in self.menu_set_default_profile.actions():
                if action.text() == profile_name:
                    self.menu_set_default_profile.removeAction(action)
                    break

    def load_current_profile(self):
        """
        Load the current profile.
        """
        logger.debug("Loading current profile.")
        self.toolBox_profile.set_profile_values(self.config.current_profile)

    def profile_change_check(self) -> bool:
        """
        Check if the current profile has unsaved changes.

        :return: True if the profile is ready to be changed, False otherwise.
        """
        logger.warning("Profile change check")
        # Check if the profile is still in the list.
        if self.comboBox_current_profile.currentText() not in list(
            self.config.saved_profiles.keys()
        ) + [cfg.DEFAULT_PROFILE_NAME]:
            logger.debug("Profile not in list.")
            # The profile is not in the list, so it must have been deleted.
            return True

        if self.toolBox_profile.is_modified():
            logger.debug(
                f"Previous profile {self.comboBox_current_profile.currentText()} has unsaved changes."
            )
            # Warn the user that he will lose unsaved changes.
            message = (
                f"The profile '{self.comboBox_current_profile.currentText()}' has unsaved changes.\n"
                f"Switching profiles will discard changes to the current profile."
            )
            response = gu.show_question(
                self,
                "Unsaved changes",
                message,
                Qw.QMessageBox.Cancel | Qw.QMessageBox.Save | Qw.QMessageBox.Discard,
            )
            if response == Qw.QMessageBox.Cancel:
                logger.debug("Profile change aborted.")
                # Abort the profile change.
                return False
            elif response == Qw.QMessageBox.Save:
                logger.debug("Saving previous profile.")
                # Save the current profile.
                self.save_profile()
            elif response == Qw.QMessageBox.Discard:
                # Continue as normal.
                logger.debug("Discarding previous profile changes.")
            else:
                raise ValueError(f"Invalid response: {response}")

        return True

    def change_current_profile(self):
        """
        Set the config option to match the current profile selector, then load it.
        """
        profile_name = self.comboBox_current_profile.currentText()
        self.config.load_profile(profile_name)
        self.load_current_profile()

    @Slot()
    def handle_profile_values_changed(self):
        """
        Handle the profile values changing.
        """
        dirty = self.toolBox_profile.is_modified()
        self.pushButton_save_profile.setEnabled(dirty)
        self.pushButton_reset_profile.setEnabled(dirty)
        self.pushButton_apply_profile.setEnabled(dirty)
        self.action_save_profile.setEnabled(dirty)

    def reset_profile(self):
        """
        Reset the current profile.
        """
        self.toolBox_profile.reset_all()
        self.handle_profile_values_changed()

    def save_profile(self, save_as: bool = False, make_new: bool = False):
        """
        Save the current profile.

        :param save_as: Whether to save as a new profile.
        :param make_new: Whether to make a new profile.
        """
        # Grab the path from the combobox's linked data. If it is none, this is the
        # default profile, so we need to save it as a new profile.
        profile_path = self.comboBox_current_profile.currentLinkedData()
        profile_name = self.comboBox_current_profile.currentText()
        if profile_path is None or make_new:
            save_as = True

        if profile_path is None and not make_new:
            # Trying to save over the default profile.
            profile_path, profile_name = self.get_new_profile_path(show_protection_hint=True)
        elif save_as:
            profile_path, profile_name = self.get_new_profile_path()

        # If the path is still none, the dialog was canceled.
        if profile_path is None:
            logger.info("User canceled profile save.")
            return

        if make_new:
            success, msg = pc.new_profile(self.config, profile_name, profile_path)
            if success:
                gu.show_info(self, "Profile Created", msg)
                self.add_new_profile_to_gui(profile_name, profile_path)
            else:
                gu.show_warning(self, "Create Error", msg)
                return
        else:
            # Proceed to write the profile.
            logger.info(f"Saving profile to {profile_path}")
            self.toolBox_profile.get_profile_values(self.config.current_profile)
            success = self.config.current_profile.write(profile_path)
            if not success:
                logger.error("Failed to save profile.")
                self.statusbar.showMessage(f"Failed to save profile to {profile_path}")
                gu.show_critical(self, "Save Error", "Failed to save profile.")
                return

            logger.info("Profile saved successfully.")
            self.statusbar.showMessage(f"Profile saved to {profile_path}")
            if save_as:
                self.config.add_profile(profile_name, profile_path)
                if not self.config.save():
                    logger.error("Failed to save config.")
                    self.statusbar.showMessage("Failed to save config.")
                    gu.show_critical(
                        self,
                        "Save Error",
                        "Failed to save the new profile to the configuration file.",
                    )
                    return
                # Add the new profile to the combobox.
                self.add_new_profile_to_gui(profile_name, profile_path)

        self.config.load_profile(profile_name)
        self.load_current_profile()
        self.handle_profile_values_changed()

    def add_new_profile_to_gui(self, profile_name: str, profile_path: Path):
        self.comboBox_current_profile.addTextItemLinkedData(profile_name, profile_path)
        self.comboBox_current_profile.setCurrentIndexByLinkedData(profile_path)
        self.menu_set_default_profile.addAction(profile_name)
        self.menu_set_default_profile.actions()[-1].triggered.connect(
            partial(self.handle_set_default_profile, profile_name)
        )
        self.menu_set_default_profile.actions()[-1].setCheckable(True)

    def get_new_profile_path(
        self, show_protection_hint: bool = False
    ) -> tuple[Path, str] | tuple[None, None]:
        """
        Open a save dialog to save the current profile.

        :param show_protection_hint: Whether to show a hint about protecting the default profile.
        """
        default_path = cu.get_default_profile_path()
        new_profile_dialog = npd.NewProfileDialog(
            default_path, show_protection_hint, cfg.RESERVED_PROFILE_NAMES
        )
        response = new_profile_dialog.exec()
        if response == Qw.QDialog.Accepted:
            return new_profile_dialog.get_save_path(), new_profile_dialog.get_name()

        return None, None

    """
    Glossary
    """

    def load_glossary(self):
        """
        Open a glossary file and parse it.
        Afterwards, notify any files to apply it.
        """
        path = Path(self.config.glossary_path)

        if not path.is_file():
            logger.warning(f"Glossary file not found: {path}")
            self.statusbar.showMessage(f"Glossary file not found.", 10_000)
            return

        # Check if we haven't loaded the file before.
        if self.glossary.is_same_glossary(path):
            logger.info("Glossary file unchanged")
            return
        # Start the worker.
        self.glossary_worker_start(path)

    def glossary_worker_start(self, path: Path):
        """
        Initialize generic QRunner worker to load a glossary file.

        :param path: Path to the glossary file.
        """
        self.statusbar.showMessage("Loading glossary...")
        worker = wt.Worker(gl.parse_glossary, path=path, no_progress_callback=True)
        worker.signals.result.connect(self.glossary_worker_result)
        worker.signals.error.connect(self.glossary_worker_error)
        logger.debug(f"Loading glossary from {path}")
        # Execute.
        self.threadpool.start(worker)

    # def glossary_worker_result(self, glossary: st.Glossary):
    #     """
    #     Notify any files to apply the glossary.
    #
    #     :param glossary: The glossary received from the worker.
    #     """
    #     self.glossary = glossary
    #     terms_found = len(glossary)
    #     self.statusbar.showMessage(
    #         f"Loaded {terms_found} {hp.f_plural(terms_found, 'term')} from glossary."
    #     )
    #     logger.info(f"Glossary loaded, {terms_found} {hp.f_plural(terms_found, 'term')} found.")
    #     logger.debug(f"Glossary dump: \n{glossary}")
    #     self.text_params_changed.emit(self.glossary)
    #
    # def glossary_worker_error(self, error: wt.WorkerError):
    #     """
    #     Notify the user of an error.
    #     """
    #     self.statusbar.showMessage(f"Error loading glossary.", 10_000)
    #     show_warning(self, "Glossary Error", f"Failed to open glossary file\n\n{error.value}")
    #     logger.error(f"Failed to open glossary file.\n{error}")
    #
    # """
    # Translation
    # """
    #
    # def start_translating(self):
    #     logger.info("Starting translation.")
    #     # Check if the API is ready.
    #     translator = self.open_translator()
    #     if translator is None:
    #         show_warning(
    #             self, "API Error", "Failed to open DeepL translator. Please check account settings."
    #         )
    #     if self.config.tl_mock:
    #         show_warning(
    #             self,
    #             "Mock Mode",
    #             "Translations are being performed with the mock translator.\nDo not expect accurate results.",
    #         )
    #     else:
    #         # Skip the char count warning when mocking because then we can't load usage stats and it doesn't matter.
    #         if not self.char_count_warning(translator):
    #             return
    #
    #     # Restrict user interaction.
    #     self.processing = True
    #     self.update_input_buttons()
    #     self.set_translation_parameters_enabled(False)
    #     self.abort_button_enabled(True)
    #     self.show_button_abort()
    #     self.progressBar.setValue(0)
    #     self.show_progress()
    #     self.label_progress.setText("Preparing...")
    #     self.statusbar.showMessage("Translating...")
    #
    #     # Send the data off to the worker.
    #     worker = ai.DeeplWorker(
    #         translator=translator, input_files=self.file_table.files, config=self.config
    #     )
    #     worker.signals.result.connect(self.translation_worker_result)
    #     worker.signals.progress.connect(self.translation_worker_progress)
    #     worker.signals.error.connect(self.translation_worker_error)
    #     self.abort_translation_worker.connect(worker.abort)
    #     self.threadpool.start(worker)
    #
    # def char_count_warning(self, translator: deepl.Translator) -> bool:
    #     # Make sure the user is aware of how many characters will be translated.
    #     # Check if the allotted character count won't exceed the quota.
    #     total_chars = sum(file.char_count for file in self.file_table.files.values())
    #     api_usage = translator.get_usage()
    #     allowed_chars = api_usage.character.limit - api_usage.character.count
    #     remaining_chars = allowed_chars - total_chars
    #     warning_msg = (
    #         f"You are about to translate {hp.format_char_count(total_chars)} "
    #         f"{hp.f_plural(total_chars, 'character')},\n"
    #         f"leaving you with {hp.format_char_count(remaining_chars)} "
    #         f"{hp.f_plural(remaining_chars, 'character')}."
    #         f"\nProceed?"
    #     )
    #     if api_usage.character.limit_reached:
    #         warning_msg = "You have reached your character limit.\nProceed anyway?"
    #     elif total_chars > allowed_chars:
    #         warning_msg = (
    #             f"You are about to translate {hp.format_char_count(total_chars)} "
    #             f"{hp.f_plural(total_chars, 'character')}, "
    #             f"which exceeds your remaining character limit of {hp.format_char_count(allowed_chars)}.\nProceed anyway?"
    #         )
    #     # Ask the user if he wants to proceed, just in case.
    #     logger.info(f"Character limit warning: {warning_msg}")
    #     affirmation = show_question(self, "API Limit", warning_msg)
    #     if not affirmation:
    #         logger.info("Translation cancelled.")
    #         return False
    #     return True
    #
    # def translation_worker_result(self, exit_code: ai.State):
    #     """
    #     If we made it here, the translation was either successful or aborted.
    #     Notify the user, write the output, and reset the ui.
    #
    #     :param exit_code: The exit code of the translation.
    #     """
    #
    #     logger.info(f"Translation finished with exit code {exit_code}")
    #     self.text_output_changed.emit()
    #
    #     if exit_code == ai.State.DONE:
    #         self.statusbar.showMessage("Translation finished.")
    #         show_info(self, "Finished", "Translations successfully completed.")
    #         for file_id in self.file_table.files:
    #             self.write_output_file(file_id)
    #
    #     elif exit_code == ai.State.ABORTED:
    #         self.statusbar.showMessage("Translation aborted.")
    #         show_info(self, "Aborted", "Translation aborted.")
    #         if self.config.dump_on_abort:
    #             for file_id in self.file_table.files:
    #                 self.write_output_file(file_id)
    #
    #     elif exit_code == ai.State.QUOTA_EXCEEDED:
    #         self.statusbar.showMessage("API quota exceeded.")
    #         show_warning(
    #             self,
    #             "API Quota Exceeded",
    #             "The DeepL API quota has been exceeded.\nDumping output files.",
    #         )
    #         for file_id in self.file_table.files:
    #             self.write_output_file(file_id)
    #
    #     self.translation_worker_finished()
    #
    # def translation_worker_progress(
    #     self, key: str, message: str, processed_chars: int | None, total_chars: int | None
    # ):
    #     """
    #     Update the file with the key in the file table to display this message.
    #
    #     :param key: InputFile ID.
    #     :param message: Progress status.
    #     :param processed_chars: Number of characters processed.
    #     :param total_chars: Total number of characters to process.
    #     """
    #     logger.debug(f"Translation progress: {key} {message} {processed_chars} {total_chars}")
    #     self.file_table.show_file_progress(key, message)
    #     if processed_chars is not None and total_chars is not None:
    #         self.update_translation_status(processed_chars, total_chars)
    #
    # def translation_worker_error(self, error: wt.WorkerError):
    #     logger.error(f"Translation failed.\n{error}")
    #     self.statusbar.showMessage(f"Translation failed.")
    #     show_warning(self, "Translation Error", f"Translation failed.\n\n{error.value}")
    #     self.translation_worker_finished()
    #
    # def translation_worker_finished(self):
    #     """
    #     Update the UI to reflect the finished state.
    #     """
    #     self.processing = False
    #     self.update_input_buttons()
    #     self.set_translation_parameters_enabled(True)
    #     self.abort_button_enabled(False)
    #     self.show_button_start()
    #     self.hide_progress()
    #     self.config.save()  # Save the last average time/1000 characters.
    #     self.load_config_to_ui()
    #
    # def abort_translating(self):
    #     logger.info("Aborting translation.")
    #     self.abort_translation_worker.emit()
    #     self.statusbar.showMessage("Aborting translation...")
    #
    # def write_output_file(self, file_id: str):
    #     """
    #     Write the output file to disk.
    #     Check the type of the file (text or epub) and run the appropriate function.
    #
    #     :param file_id: InputFile ID.
    #     """
    #
    #     file = self.file_table.files[file_id]
    #     try:
    #         if isinstance(file, st.TextFile):
    #             self.write_output_text_file(file_id)
    #         elif isinstance(file, st.EpubFile):
    #             self.write_output_epub_file(file_id)
    #         else:
    #             raise TypeError(f"Unknown file type: {file}")
    #
    #     except OSError as e:
    #         path_out = make_output_filename(file, self.config)
    #         logger.error(f"Failed to write translation to {path_out}.\n{e}\n\n")
    #         show_warning(
    #             self,
    #             "Output Error",
    #             f"Failed to write translation to {path_out}.\n\n{e}\n\n"
    #             f"Please make sure you have write permissions to the output directory.\n"
    #             f"To save the translation in another location without needing to re-translate, "
    #             f'select your file, then click the "preview" button.\n'
    #             f'There, select the translated version and press the "Save Preview to File" button.',
    #         )
    #         self.file_table.show_file_progress(file_id, "Could not write output!")
    #
    # def write_output_text_file(self, file_id: str):
    #     """
    #     Write the translation of a file to the assigned output path.
    #
    #     Check the status of the file. We decide 3 cases:
    #     1. File is not translated.
    #        - Skip this file.
    #     2. File is translated.
    #        - Write the translation to the output path.
    #     3. File is partially translated.
    #        - Dump the translation to the output path, if so configured.
    #
    #     :param file_id: InputFile ID.
    #     """
    #     file = self.file_table.files[file_id]
    #     text_out = file.get_translated_text()
    #     path_out = make_output_filename(file, self.config)
    #
    #     if text_out is None:  # Case 1.
    #         logger.info(f"Skipping file {file_id} because it has not been translated.")
    #         self.file_table.show_file_progress(file_id, "Not translated.")
    #         return
    #     else:
    #         # Ensure the output directory exists.
    #         path_out.parent.mkdir(parents=True, exist_ok=True)
    #
    #         with open(path_out, "w", encoding="utf-8") as f:
    #             f.write(text_out)
    #             logger.info(f"Wrote translation to {path_out}")
    #             if file.translation_incomplete():
    #                 self.file_table.show_file_progress(file_id, "Incomplete output written.")
    #             else:
    #                 self.file_table.show_file_progress(file_id, "Translation written.")
    #
    # def write_output_epub_file(self, file_id: str):
    #     """
    #     Write the translation of a file to the assigned output path.
    #
    #     Check the status of the file. We decide 3 cases:
    #     1. File is not translated.
    #        - Skip this file.
    #     2. File is translated.
    #        - Write the translation to the output path.
    #     3. File is partially translated.
    #        - Dump the translation to the output path, if so configured.
    #
    #     :param file_id: InputFile ID.
    #     """
    #     file: st.EpubFile = self.file_table.files[file_id]
    #     path_out = make_output_filename(file, self.config)
    #
    #     if not file.translation_incomplete() and not file.is_translated():
    #         # Skip this because we have nothing to dump.
    #         logger.info(f"Skipping file {file_id} because nothing has been translated.")
    #         self.file_table.show_file_progress(file_id, "Not translated.")
    #         return
    #
    #     file.write(st.ProcessLevel.TRANSLATED, path_out)

    """
    Log file
    """

    def set_up_statusbar(self):
        """
        Add a label to show the current char total and time estimate.
        Add a flat button to the statusbar to offer opening the config file.
        Add a flat button to the statusbar to offer opening the log file.
        """
        self.statusbar.setSizeGripEnabled(False)
        self.statusbar.setContentsMargins(0, 0, 6, 0)

        self.label_stats = Qw.QLabel("")
        self.statusbar.addPermanentWidget(self.label_stats)

        button_config = Qw.QPushButton("Open Config")
        button_config.clicked.connect(partial(hp.open_file, cu.get_config_path()))
        button_config.setFlat(True)
        self.statusbar.addPermanentWidget(button_config)

        button_log = Qw.QPushButton("Open Log")
        button_log.clicked.connect(partial(hp.open_file, cu.get_log_path()))
        button_log.setFlat(True)
        self.statusbar.addPermanentWidget(button_log)

    """
    Simple UI manipulation functions
    """

    def hide_progress(self):
        self.widget_progress_drawer.hide()

    def show_progress(self):
        self.widget_progress_drawer.show()

    def fixed_output_dir_enabled(self, enabled: bool):
        self.lineEdit_file_out_dir.setEnabled(enabled)
        self.pushButton_file_dir_browse.setEnabled(enabled)

    def glossary_enabled(self, enabled: bool):
        self.lineEdit_glossary_file.setEnabled(enabled)
        self.pushButton_glossary_file_browse.setEnabled(enabled)

    def start_button_enabled(self, enabled: bool):
        self.pushButton_start.setEnabled(enabled)

    def abort_button_enabled(self, enabled: bool):
        self.pushButton_abort.setEnabled(enabled)

    def show_button_start(self):
        self.pushButton_start.show()
        self.pushButton_abort.hide()

    def show_button_abort(self):
        self.pushButton_start.hide()
        self.pushButton_abort.show()

    def show_api_status(self, good: bool):
        self.label_api_status_good.setVisible(good)
        self.label_api_status_good_icon.setVisible(good)
        self.label_api_status_bad.setVisible(not good)
        self.label_api_status_bad_icon.setVisible(not good)

    # def update_input_buttons(self):
    #     logger.debug("Updating input buttons")
    #
    #     if self.processing:
    #         self.action_Open_Files.setEnabled(False)
    #         self.action_Clear_Files.setEnabled(False)
    #     else:
    #         self.action_Open_Files.setEnabled(True)
    #         self.action_Clear_Files.setEnabled(self.file_table.rowCount() > 0)
    #
    # def update_current_usage(self, translator: deepl.Translator | None):
    #     if translator is None:
    #         self.label_api_usage.setText("—")
    #         self.label_api_usage_warn_icon.hide()
    #         self.label_api_usage_error_icon.hide()
    #         return
    #
    #     usage = translator.get_usage()
    #     count = usage.character.count
    #     limit = usage.character.limit
    #     percentage = count / limit * 100  # Output with 2 decimals.
    #
    #     count_str = hp.format_char_count(count)
    #     limit_str = hp.format_char_count(limit)
    #
    #     self.label_api_usage.setText(f"{percentage:.2f}%  {count_str} / {limit_str} characters")
    #
    #     if percentage < 90:
    #         self.label_api_usage_warn_icon.hide()
    #         self.label_api_usage_error_icon.hide()
    #     elif percentage < 100:
    #         self.label_api_usage_warn_icon.show()
    #         self.label_api_usage_error_icon.hide()
    #     else:
    #         self.label_api_usage_warn_icon.hide()
    #         self.label_api_usage_error_icon.show()
    #
    # def ready_to_translate(self):
    #     logger.info("Ready to translate.")
    #     if self.threadpool.activeThreadCount() > 0:
    #         logger.error("Threadpool still has active threads.")
    #     self.start_button_enabled(True)
    #     self.show_button_start()
    #     self.statusbar.showMessage("Ready to translate.")
    #
    # def not_ready_to_translate(self):
    #     logger.info("Not ready to translate.")
    #     self.start_button_enabled(False)
    #     self.show_button_start()
    #
    # def set_translation_parameters_enabled(self, enabled: bool):
    #     """
    #     When processing, language, output, glossary, and account settings must all be locked.
    #
    #     :param enabled: Sets the enabled flag for all relevant widgets accordingly.
    #     """
    #     self.groupBox_language.setEnabled(enabled)
    #     self.groupBox_glossary.setEnabled(enabled)
    #     self.groupBox_api.setEnabled(enabled)
    #     self.checkBox_file_fixed_dir.setEnabled(enabled)
    #     self.lineEdit_file_out_dir.setEnabled(enabled)
    #     self.pushButton_file_dir_browse.setEnabled(enabled)
    #
    # def recalculate_char_total(self):
    #     """
    #     Calculate a new total of characters to translate and display it in the status bar.
    #     """
    #     char_total = sum(file.char_count for file in self.file_table.files.values())
    #     if char_total == 0:
    #         self.label_stats.setText("")
    #         return
    #
    #     time_total = ceil(self.config.avg_time_per_mille * char_total / 1000)
    #     char_text = hp.format_char_count(char_total) + hp.f_plural(char_total, " character")
    #     time_text = f"Approx. {hp.f_time(time_total)}"
    #     self.label_stats.setText(char_text + "   " + time_text)
    #
    # def update_translation_status(self, processed_chars: int, char_total: int):
    #     """
    #     Update the translation status label.
    #     """
    #     if char_total == 0:
    #         logger.info("Total character count is 0. Nothing to do.")
    #         return
    #     char_text = hp.format_char_count(processed_chars) + " / " + hp.format_char_count(char_total)
    #     time_total = ceil(self.config.avg_time_per_mille * (char_total - processed_chars) / 1000)
    #     self.label_progress.setText(
    #         f"Translated {char_text} {hp.f_plural(char_total, 'character')}\n"
    #         f"Approximately {hp.f_time(time_total)} remaining"
    #     )
    #     self.progressBar.setValue(processed_chars / char_total * 100)
    #
    # def show_glossary_help(self):
    #     """
    #     Show the glossary documentation in a web browser.
    #     Open the github page for this.
    #     """
    #     show_info(
    #         self,
    #         "Glossary info",
    #         # language=HTML
    #         """<html>
    #                 <head/>
    #                 <body>
    #                     <p> DeepQt uses glossary files to pre-process files before sending them to the API;
    #                         this is not the same as DeepL's glossary functions. Therefore, they can be used
    #                         with any language and offer special features, which DeepL's glossaries cannot
    #                         offer.
    #                     </p>
    #                     <p>
    #                        The format of these glossaries is outlined in the
    #                         <a href="https://github.com/VoxelCubes/DeepQt/blob/master/docs/glossary_help.md">
    #                             online documentation
    #                         </a>
    #                         .
    #                     </p>
    #                 </body>
    #             </html>""",
    #     )



from enum import IntEnum
from functools import partial
from pathlib import Path
from uuid import uuid4

import PySide6.QtCore as Qc
import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw
from PySide6.QtCore import Slot
from logzero import logger

import pcleaner.config as cfg
from .CustomQ.CTableWidget import CTableWidget


class Column(IntEnum):
    ID = 0
    FILENAME = 1
    STATUS = 2
    CHARS = 3
    OUTPUT = 4


# noinspection PyUnresolvedReferences
class FileTable(CTableWidget):
    """
    Extends the functionality with custom helpers
    """

    # config: cfg.Config  # Reference to the MainWindow's config.
    # files: dict[str, st.TextFile | st.EpubFile]

    # request_text_param_update = Qc.Signal()
    # ready_for_translation = Qc.Signal()
    # not_ready_for_translation = Qc.Signal()
    # statusbar_message = Qc.Signal(str, int)
    # recalculate_char_total = Qc.Signal()

    def __init__(self, parent=None):
        CTableWidget.__init__(self, parent)

        self.files = {}
        self.threadpool = Qc.QThreadPool.globalInstance()
        self.finished_drop.connect(lambda: self.request_text_param_update.emit())

        # Make icons larger so the epub covers are more visible.
        self.setIconSize(Qc.QSize(32, 32))

    def set_config(self, config: cfg.Config):
        self.config = config

    def handleDrop(self, path: str):
        logger.debug(f"Dropped {path}")
        self.add_file(Path(path))

    def add_file(self, path: Path):
        logger.info(f"Added {path}")
        # Make sure the file is not already in the table.
        paths_in_table = [file.path for file in self.files.values()]
        if path in paths_in_table:
            logger.warning(f"File {path} already in table.")
            hp.show_warning(self, "Duplicate file", f"File {path} is already in the table.")
            return

        self.not_ready_for_translation.emit()
        file_id = str(uuid4())

        try:
            self.files[file_id] = self.initialize_file(path=path)
        except (OSError, ValueError) as e:
            # The ValueError is raised when the epub file isn't valid.
            logger.error(f"Failed to add file {path}")
            logger.error(e)
            hp.show_warning(None, "Failed to add file", f"Failed to add file {path}\n\n{e}")
            return

        # Add the new file to the table.
        file = self.files[file_id]
        self.appendRow(
            file_id,
            path.name,
            "File added",
            hp.format_char_count(file.char_count),
            str(make_output_filename(file, self.config)),
            select_new=True,
        )
        # Add icon to the filename column.
        logger.debug(f"Adding icon to {file_id}")
        if isinstance(file, st.TextFile):
            self.item(self.rowCount() - 1, Column.FILENAME).setIcon(
                Qg.QIcon.fromTheme("text-x-generic")
            )
        else:
            self.item(self.rowCount() - 1, Column.FILENAME).setIcon(
                Qg.QIcon.fromTheme("application-epub+zip")
            )
        # Align Char count to the right.
        self.item(self.rowCount() - 1, Column.CHARS).setTextAlignment(
            Qg.Qt.AlignRight | Qg.Qt.AlignVCenter
        )


#     @staticmethod
#     def initialize_file(path: Path) -> st.TextFile | st.EpubFile:
#         """
#         Read and populate the basic information of the file.
#         Text files (utf8) and epub files are supported.
#
#         :param path: The path to the file.
#         """
#         logger.debug(f"Initializing file {path}")
#         if path.suffix.lower() == ".epub":
#             return st.EpubFile(path=path, cache_dir=cfg.epub_cache_path())
#         else:
#             return st.TextFile(path=path)
#
#     @Slot()
#     def update_all_output_filenames(self):
#         """
#         Update all output filenames in the table.
#         The file names need to match the output directory preference and target language.
#         Don't update locked files.
#         """
#         for row in range(self.rowCount()):
#             file_id = self.item(row, Column.ID).text()
#             file = self.files[file_id]
#             if file.locked:
#                 continue
#             new_output_filename = make_output_filename(file, self.config)
#             self.item(row, Column.OUTPUT).setText(str(new_output_filename))
#
#         logger.debug("All output filenames updated.")
#
#     """
#     Text Processing
#     """
#
#     @Slot(st.Glossary)
#     def update_all_text_params(self, glossary: st.Glossary):
#         """
#         Update all text file parameters.
#         This means processing the glossary and quote protection, if so configured.
#         Also apply the glossary to epub files.
#         """
#         # Abort if no text files.
#         if self.rowCount() == 0:
#             return
#
#         # Try again in 0.5 seconds if the threadpool is busy.
#         if self.threadpool.activeThreadCount() > 0:
#             self.statusbar_message.emit("Waiting for previous threads to finish...", 1000)
#             Qc.QTimer.singleShot(500, partial(self.update_all_text_params, glossary))
#             return
#
#         self.statusbar_message.emit("Processing...", 5000)
#
#         # Show a progress message for all files.
#         for row in range(self.rowCount()):
#             # If the file is locked, skip it.
#             if not self.files[self.item(row, Column.ID).text()].locked:
#                 self.item(row, Column.STATUS).setText("Processing...")
#
#         logger.debug("Updating all text params")
#         for row in range(self.rowCount()):
#             self.update_file_params(row, glossary)
#
#     def update_file_params(self, row: int, glossary: st.Glossary):
#         """
#         Update the text file parameters for the given row.
#         This means processing the glossary and quote protection, if so configured.
#
#         :param row: The row in the table to update.
#         :param glossary: The glossary to apply.
#         """
#         logger.debug(f"Updating text parameters for {row}")
#         file_id = self.item(row, Column.ID).text()
#         file = self.files[file_id]
#         file_is_epub = isinstance(file, st.EpubFile)
#         file_needs_preprocessing = file_is_epub and not file.initialized
#
#         # If not processing, check if the label should be updated to say that changes were reverted.
#         if (
#             not file_needs_preprocessing
#             and not (self.config.use_glossary and glossary.is_valid())
#             and (not self.config.use_quote_protection or file_is_epub)
#         ):
#             if file.process_level != st.ProcessLevel.RAW:
#                 file.process_level = st.ProcessLevel.RAW
#                 self.item(row, Column.STATUS).setText("Reset to original")
#                 self.recalculate_char_count(file_id)
#                 return
#
#         if self.config.use_glossary and glossary.is_valid() and glossary.hash != file.glossary_hash:
#             glossary_to_pass = glossary
#         else:
#             glossary_to_pass = None
#
#         # Test and set lock.
#         if file.locked:
#             logger.warning(f"File {file.path} is locked, access denied.")
#             return
#         file.locked = True
#
#         # Crunch time begins for the worker. Bless his soul.
#         # (Move this to another thread because it's CPU intensive.)
#         if not file_is_epub:
#             # Start the text file worker.
#             worker = wt.Worker(
#                 self.text_process_work,
#                 file_id=file_id,
#                 text_file=file,
#                 glossary=glossary_to_pass,
#                 apply_glossary=self.config.use_glossary and glossary.is_valid(),
#                 apply_protection=self.config.use_quote_protection,
#             )
#             logger.debug(
#                 f"Worker Thread processing text file {file.path}: "
#                 f"Glossary: {glossary_to_pass is not None} | Protection: {self.config.use_quote_protection}"
#             )
#         else:
#             # Start the epub file worker.
#             worker = wt.Worker(
#                 self.epub_process_work,
#                 file_id=file_id,
#                 epub_file=file,
#                 glossary=glossary_to_pass,
#                 apply_glossary=self.config.use_glossary and glossary.is_valid(),
#             )
#             logger.debug(
#                 f"Worker Thread processing epub file {file.path}: "
#                 f"Glossary: {glossary_to_pass is not None}"
#             )
#
#         worker.signals.result.connect(self.file_process_worker_result)
#         worker.signals.progress.connect(self.file_process_worker_progress)
#         worker.signals.error.connect(self.file_process_worker_error)
#         worker.signals.finished.connect(self.file_process_worker_finished)
#         # Execute.
#         logger.info(f"Executing worker thread {file.path}")
#         self.threadpool.start(worker)
#
#     """
#     Workers
#     """
#
#     @staticmethod
#     def text_process_work(
#         file_id: str,
#         text_file: st.TextFile,
#         glossary: st.Glossary,
#         apply_glossary: bool,
#         apply_protection: bool,
#         progress_callback: Qc.Signal,
#     ):
#         """
#         Apply the glossary to the given text file.
#
#         :param file_id: The ID of the file to process.
#         :param text_file: The text file to apply the glossary to.
#         :param glossary: The glossary to apply. None if no glossary is to be applied.
#         :param apply_glossary: True if the glossary is to be applied.
#         :param apply_protection: Whether to apply quote protection.
#         :param progress_callback: A callback to call with the progress of the processing.
#         """
#
#         if apply_glossary:
#             progress_callback.emit((file_id, "Applying glossary..."))
#             if (
#                 glossary is not None
#             ):  # In this case, the glossary was already applied and still cached.
#                 text_file.text_glossary = gls.process_text(text_file.text, glossary)
#                 text_file.glossary_hash = glossary.hash
#             # Set it either way, so that the file knows it's been processed.
#             text_file.process_level = st.ProcessLevel.GLOSSARY
#
#             if apply_protection:
#                 progress_callback.emit((file_id, "Applying EQP next..."))
#                 text_file.text_glossary_protected = qp.protect_text(text_file.text_glossary)
#                 text_file.process_level = st.ProcessLevel.GLOSSARY_PROTECTED
#         elif apply_protection:
#             progress_callback.emit((file_id, "Applying EQP..."))
#             text_file.text_protected = qp.protect_text(text_file.text)
#             text_file.process_level = st.ProcessLevel.PROTECTED
#
#         progress_callback.emit((file_id, "Ready"))
#
#         # Return the file id to update the table.
#         logger.info(f"Text file {text_file.path} processed.")
#         return file_id
#
#     def epub_process_work(
#         self,
#         file_id: str,
#         epub_file: st.EpubFile,
#         glossary: st.Glossary,
#         apply_glossary: bool,
#         progress_callback: Qc.Signal,
#     ):
#         """
#         Apply the glossary to the given epub file.
#
#         :param file_id: The ID of the file to process.
#         :param epub_file: The epub file to apply the glossary to.
#         :param glossary: The glossary to apply. None if no glossary is to be applied.
#         :param apply_glossary: True if the glossary is to be applied.
#         :param progress_callback: A callback to call with the progress of the processing.
#         """
#
#         # Pre-process the epub file.
#         progress_callback.emit((file_id, "Loading epub..."))
#         epub_file.initialize_files(
#             nuke_ruby=self.config.epub_nuke_ruby,
#             nuke_kobo=self.config.epub_nuke_kobo,
#             nuke_indents=self.config.epub_nuke_indents,
#             crush_html=self.config.epub_crush,
#             make_text_horizontal=self.config.epub_make_text_horizontal,
#             ignore_empty=self.config.epub_ignore_empty_html,
#         )
#
#         if apply_glossary:
#             progress_callback.emit((file_id, "Applying glossary..."))
#             if (
#                 glossary is not None
#             ):  # In this case, the glossary was already applied and still cached.
#                 gls.process_epub_file(epub_file, glossary)
#             # Set it either way, so that the file knows it's been processed.
#             epub_file.process_level = st.ProcessLevel.GLOSSARY
#
#         progress_callback.emit((file_id, "Ready"))
#
#         # Return the file id to update the table.
#         logger.info(f"Epub file {epub_file.path} processed.")
#         return file_id
#
#     """
#     Worker Callbacks
#     """
#
#     def file_process_worker_result(self, file_id: str):
#         """
#         Update the table with the result of the glossary processing.
#         """
#         logger.debug(f"Worker thread {file_id} finished.")
#         # Ignore if the file no longer exists.
#         if file_id not in self.files:
#             logger.info(f"File {file_id} no longer exists, ignoring result.")
#             return
#
#         self.recalculate_char_count(file_id)
#         # Try to add the cover as the icon, if this was an epub.
#         file = self.files[file_id]
#         if isinstance(file, st.EpubFile) and file.cover_image is not None:
#             absolute_path = file.cache_dir / file.cover_image
#             try:
#                 row = self.findItems(file_id, Qc.Qt.MatchExactly)[0].row()
#                 self.item(row, Column.FILENAME).setIcon(Qg.QIcon(str(absolute_path)))
#                 logger.info(f"Set cover image for {file_id} to {absolute_path}")
#                 file.cover_image = None  # Clear the cover image so it's not loaded again.
#             except OSError as e:
#                 logger.error(f"Could not load cover image {absolute_path}")
#                 logger.exception(e)
#
#     def file_process_worker_progress(self, progress: tuple[str, str]):
#         """
#         Update the progress bar in the table.
#         Unwrap the tuple. This is just because worker signals only transmit 1 object.
#
#         :param progress: The progress tuple: (file_id, message)
#         """
#         file_id, message = progress
#         # If the file_id no longer exists, ignore.
#         if file_id not in self.files:
#             logger.info(f"Worker progress for file {file_id} ignored, as it no longer exists.")
#             return
#         self.show_file_progress(file_id, message)
#
#     def file_process_worker_error(self, error: wt.WorkerError):
#         """
#         Display an error message in the table.
#         """
#         # Extract the row from the WorkerError's kwargs.
#         file_id = error.kwargs["file_id"]
#         file = self.files[file_id]
#         logger.error(f"Failed to process {file.path.name}\n{error}")
#         self.update_table_cell(file_id, Column.STATUS, "Failed to process.")
#
#     def file_process_worker_finished(self, initial_args: tuple[list, dict]):
#         """
#         Unlock the file after processing is finished.
#         """
#         args, kwargs = initial_args
#         file_id = kwargs["file_id"]
#         try:
#             text_file = self.files[file_id]
#         except KeyError:
#             # The file was removed from the table before processing finished.
#             logger.info(
#                 f"File {file_id} was removed from the table before processing finished. Ignoring worker results."
#             )
#             return
#
#         text_file.locked = False
#         logger.debug(f"Worker thread {text_file.path} finished.")
#         if self.all_files_ready():
#             self.ready_for_translation.emit()
#
#     """
#     Misc.
#     """
#
#     def update_table_cell(self, file_id: str, column: int, message: str):
#         """
#         Show the translation progress in the table.
#         """
#         self.item(self.findItems(file_id, Qc.Qt.MatchExactly)[0].row(), column).setText(message)
#
#     def show_file_progress(self, file_id: str, message: str):
#         """
#         Show the translation progress in the table.
#         """
#         self.update_table_cell(file_id, Column.STATUS, message)
#
#     def all_files_ready(self) -> bool:
#         """
#         Check if all files' process level matches the expected value and are not locked.
#         If no files exist, return False.
#         Note: Epub files don't use quote protection, so ignore that option.
#
#         :return: True if all files are ready.
#         """
#         expected_process_level_text = st.ProcessLevel.RAW
#         expected_process_level_epub = st.ProcessLevel.RAW
#         if self.config.use_glossary:
#             expected_process_level_text |= st.ProcessLevel.GLOSSARY
#             expected_process_level_epub |= st.ProcessLevel.GLOSSARY
#         if self.config.use_quote_protection:
#             expected_process_level_text |= st.ProcessLevel.PROTECTED
#
#         if self.rowCount() == 0:
#             return False
#
#         for row in range(self.rowCount()):
#             file = self.files[self.item(row, Column.ID).text()]
#
#             if file.locked:
#                 return False
#
#             if isinstance(file, st.TextFile):
#                 if (
#                     self.files[self.item(row, Column.ID).text()].process_level
#                     != expected_process_level_text
#                 ):
#                     return False
#             else:
#                 if (
#                     self.files[self.item(row, Column.ID).text()].process_level
#                     != expected_process_level_epub
#                 ):
#                     return False
#         return True
#
#     def browse_add_file(self):
#         """
#         Browse for a file and add it to the table.
#         Supported file types: txt and epub.
#         """
#         path = Qw.QFileDialog.getOpenFileName(
#             self, "Select file", filter="Text or Epub (*.txt *.epub)"
#         )[0]
#         if path:
#             self.add_file(Path(path))
#             self.request_text_param_update.emit()
#
#     def preview_selected_file(self):
#         """
#         Open the preview window for the selected file.
#         """
#         selected_row = self.selectedItems()[0].row()
#         file_id = self.item(selected_row, Column.ID).text()
#         file = self.files[file_id]
#         if isinstance(file, st.TextFile):
#             dtp.TextPreview(self, file, self.config).exec()
#         else:
#             dep.EpubPreview(self, file, self.config).exec()
#
#     def remove_selected_file(self):
#         """
#         Remove the selected file from the table.
#         """
#         selected_row = self.selectedItems()[0].row()
#         file_id = self.item(selected_row, Column.ID).text()
#         self.removeRow(selected_row)
#         # This isn't automatically emitted when removing a row, but is necessary to update the buttons.
#         self.itemSelectionChanged.emit()
#
#         del self.files[file_id]
#         if self.all_files_ready():
#             self.ready_for_translation.emit()
#         else:
#             self.not_ready_for_translation.emit()
#         self.recalculate_char_total.emit()
#
#     def remove_all_files(self):
#         """
#         Remove all files from the table.
#         """
#         self.clearAll()
#         self.files.clear()
#         self.not_ready_for_translation.emit()
#         self.recalculate_char_total.emit()
#
#     def recalculate_char_count(self, file_id: str):
#         """
#         Update the table after a file has been processed.
#
#         :param file_id: The ID of the file to update.
#         """
#         file = self.files[file_id]
#         char_count = file.char_count
#         logger.debug(f"Recalculating char count for {file.path.name} ({char_count} chars).")
#         self.update_table_cell(file_id, Column.CHARS, hp.format_char_count(char_count))
#         self.recalculate_char_total.emit()
#
#
# def make_output_filename(input_file: st.InputFile, config: cfg.Config) -> Path:
#     # Append the language code to the file stem.
#     path = input_file.path
#     # Add lang extension.
#     path = path.with_stem(f"{path.stem}_{config.lang_to.lower()}")
#     # Add dump extension if translation failed.
#     if input_file.translation_incomplete():
#         path = path.with_suffix(".DUMP")
#
#     if config.use_fixed_output_path:
#         # Make the fixed output path absolute, meaning it isn't relative to the
#         # current working directory.
#         path = Path("/") / config.fixed_output_path / path.name
#         path = path.resolve()
#
#     path = hp.ensure_unique_file_path(path)
#
#     return path



from pathlib import Path

from PIL import Image
from logzero import logger, loglevel, DEBUG, INFO

import pcleaner.image_ops as ops
import pcleaner.structures as st


def clean_page(m_data: st.MaskerData) -> list[tuple[Path, bool, int, float]]:
    """
    Do all the shit and return analytics for mask fitting.

    :param m_data: All the data needed for the cleaning process.
    :return: Analytics, consisting of the json file path, and a list of whether the mask was generated, the index of
        the best mask, and the border uniformity of the best mask for each box.
    """

    page_data = st.PageData.from_json(m_data.json_path.read_text())

    # Make a shorter alias.
    g_conf = m_data.general_config
    m_conf = m_data.masker_config

    original_path = Path(page_data.original_path)
    original_img_path_as_png = original_path.with_suffix(
        ".png"
    )  # Make sure all derived file names are .png.
    # Clobber protection prefixes have the form "[A-Z]{4}-\d+_file name", ex. JMCF-0_0023.json
    clobber_protection_prefix = m_data.json_path.stem.split("_")[0]
    cache_out_path = (
        m_data.cache_dir / f"{clobber_protection_prefix}_{original_img_path_as_png.name}"
    )

    def save_mask(img, name_suffix):
        if m_data.show_masks:
            img.save(cache_out_path.with_stem(cache_out_path.stem + name_suffix))

    # Load the base image.
    base_image = Image.open(page_data.image_path)

    # Make a box mask. These will serve as basic masks as opposed to the AI generated precise ones.
    box_mask = page_data.make_box_mask(base_image.size, st.BoxType.EXTENDED_BOX)

    save_mask(box_mask, "_box_mask")

    # Load the precise AI-generated mask.
    mask = Image.open(page_data.mask_path)
    mask = mask.convert("1", dither=Image.NONE)  # Convert to bitmap.

    # Cut the mask to the box mask, leaving only the intersection.
    cut_mask = ops.mask_intersection(mask, box_mask)

    save_mask(cut_mask, "_cut_mask")

    # Use the merged extended boxes to guard against overlapping boxes after having extended them.
    # Generate varying sizes of the precise mask and then rank them based on border uniformity.
    mask_fitments: list[st.MaskFittingResults] = [
        ops.pick_best_mask(
            base=base_image,
            precise_mask=cut_mask,
            box_mask=box_mask,
            masking_box=masking_box,
            reference_box=reference_box,
            masker_conf=m_conf,
            scale=page_data.scale,
            save_masks=m_data.show_masks,
            analytics_page_path=Path(original_img_path_as_png),
        )
        for masking_box, reference_box in zip(
            page_data.merged_extended_boxes, page_data.reference_boxes
        )
    ]
    # Take out all entries that are None, since these masks were false positives.
    mask_fitments = [m for m in mask_fitments if m is not None]
    # Take out the analytics.
    # The analytics consist of (mask could be found: bool, mask index: int, border deviation: float)
    analytics: list[tuple[Path, bool, int, float]] = [m.analytics for m in mask_fitments]

    # Remove entries if they failed.
    best_masks = [m for m in mask_fitments if not m.failed]

    combined_mask = ops.combine_best_masks(base_image.size, best_masks)

    if m_data.show_masks:
        # Save the masks in the debug folder.
        ops.visualize_mask_fitments(
            base_image, mask_fitments, cache_out_path.with_stem(cache_out_path.stem + "_masks")
        )

        # Output the result with the debug filter.
        combined_mask_debug = ops.apply_debug_filter_to_mask(combined_mask, m_conf.debug_mask_color)
        base_image_copy = base_image.copy()
        base_image_copy.paste(combined_mask_debug, (0, 0), combined_mask_debug)
        save_mask(base_image_copy, "_with_masks")

    # Save the combined mask for denoising.
    combined_mask_path = cache_out_path.with_stem(cache_out_path.stem + "_combined_mask")
    combined_mask.save(combined_mask_path)
    save_denoising_data(
        Path(page_data.original_path),
        original_img_path_as_png,
        combined_mask_path,
        Path(page_data.image_path),
        cache_out_path,
        page_data.scale,
        m_conf.mask_max_standard_deviation,
        mask_fitments,
    )

    # Settle on the final output path for the cleaned image.
    # Check if outputting directly.
    if m_data.output_dir is not None:
        # If the scale isn't 1, then we need to reload the original image and scale the mask to fit.
        if page_data.scale != 1:
            cleaned_image = Image.open(page_data.original_path)
            combined_mask = combined_mask.resize(cleaned_image.size, Image.NEAREST)
        else:
            cleaned_image = base_image.copy()

        cleaned_image.paste(combined_mask, (0, 0), combined_mask)

        if m_data.output_dir.is_absolute():
            final_out_path = m_data.output_dir / original_img_path_as_png.name
        else:
            # Take the original image path, and place the image in a subdirectory.
            # This is for when multiple directories were passed in.
            final_out_path = (
                original_img_path_as_png.parent / m_data.output_dir / original_img_path_as_png.name
            )

        final_out_path.parent.mkdir(parents=True, exist_ok=True)
        final_cleaned_out_path = final_out_path.with_name(final_out_path.stem + "_clean.png")
        final_mask_out_path = final_out_path.with_name(final_out_path.stem + "_mask.png")

        # Check what the preferred output format is.
        if g_conf.preferred_file_type is None:
            # Use the original file type.
            final_cleaned_out_path = final_cleaned_out_path.with_suffix(
                Path(page_data.original_path).suffix
            )
        else:
            final_cleaned_out_path = final_cleaned_out_path.with_suffix(g_conf.preferred_file_type)

        if g_conf.preferred_mask_file_type is None:
            # Use png by default.
            final_mask_out_path = final_mask_out_path.with_suffix(".png")
        else:
            final_mask_out_path = final_mask_out_path.with_suffix(g_conf.preferred_mask_file_type)

        # The arg parser should ensure that both can't be true at once, not like that'd be an issue, just plain silly.
        if not m_data.save_only_mask and not m_data.save_only_text:
            # Save the final image.
            logger.debug(f"Saving final image to {final_cleaned_out_path}")
            ops.save_optimized(cleaned_image, final_cleaned_out_path, original_path)

        if not m_data.save_only_cleaned and not m_data.save_only_text:
            # Save the final image.
            logger.debug(f"Saving final mask to {final_mask_out_path}")
            ops.save_optimized(combined_mask, final_mask_out_path)

        if m_data.extract_text:
            # Extract the text layer from the image.
            logger.debug(f"Extracting text from {final_cleaned_out_path}")
            text_img = ops.extract_text(base_image, combined_mask)
            text_out_path = final_out_path.with_name(final_out_path.stem + "_text.png")
            if g_conf.preferred_mask_file_type is None:
                # Use png by default.
                text_out_path = text_out_path.with_suffix(".png")
            else:
                text_out_path = text_out_path.with_suffix(g_conf.preferred_mask_file_type)
            ops.save_optimized(text_img, text_out_path, original_path)

    return analytics


def save_denoising_data(
    original_path: Path,
    target_path: Path,
    mask_path: Path,
    base_image_path: Path,
    cache_path: Path,
    scale: float,
    mask_max_deviation: float,
    mask_fitments: list[st.MaskFittingResults],
):
    """
    Save the data needed for denoising.

    :param original_path: The path to the original image.
    :param target_path: The path to the original image with png suffix.
    :param mask_path: The path to the combined mask.
    :param base_image_path: The path to the cached base image.
    :param cache_path: The path to the cache directory.
    :param scale: The scale of the original image to the cached base image.
    :param mask_max_deviation: The maximum deviation of the mask.
    :param mask_fitments: The mask fitments.
    """

    # Gather the data needed for denoising, don't include those with a deviation greater than the max deviation.
    boxes_with_deviation = [
        m.noise_mask_data for m in mask_fitments if m.analytics_std_deviation <= mask_max_deviation
    ]

    # Save the data.
    mask_data = st.MaskData(
        original_path, target_path, base_image_path, mask_path, scale, boxes_with_deviation
    )
    mask_data.write_json(cache_path.with_name(cache_path.stem + "#mask_data.json"))



import platform
import subprocess
import zipfile as zf
from pathlib import Path

import PySide6.QtWidgets as Qw
from logzero import logger


# For all show functions, pad the dialog message, so that the dialog is not too narrow for the window title.
MIN_MSG_LENGTH = 50


def show_critical(parent, title: str, msg: str):
    msg = msg.ljust(MIN_MSG_LENGTH)
    return Qw.QMessageBox.critical(parent, title, msg, Qw.QMessageBox.Yes, Qw.QMessageBox.Abort)


def show_warning(parent, title: str, msg: str):
    msg = msg.ljust(MIN_MSG_LENGTH)
    Qw.QMessageBox.warning(parent, title, msg, Qw.QMessageBox.Ok)


def show_info(parent, title: str, msg: str):
    msg = msg.ljust(MIN_MSG_LENGTH)
    Qw.QMessageBox.information(parent, title, msg, Qw.QMessageBox.Ok)


def show_question(
    parent, title: str, msg: str, buttons=Qw.QMessageBox.Yes | Qw.QMessageBox.Cancel
) -> int:
    msg = msg.ljust(MIN_MSG_LENGTH)
    dlg = Qw.QMessageBox(parent)
    dlg.setWindowTitle(title)
    dlg.setText(msg)
    dlg.setStandardButtons(buttons)
    dlg.setIcon(Qw.QMessageBox.Question)
    return dlg.exec()



import sys
import argparse
import platform

import PySide6.QtWidgets as Qw
import PySide6.QtGui as Qg
import logzero
from logzero import logger

from pcleaner.gui.mainwindow_driver import MainWindow
import pcleaner.config as cfg
import pcleaner.cli_utils as cu
from pcleaner import __display_name__, __version__


# This import is needed to load the icons.
import pcleaner.gui.rc_generated_files.icons_rc
import pcleaner.gui.rc_generated_files.theme_icons_rc


def launch(input_paths: list[str]) -> None:
    """
    Launch the GUI.

    :param input_paths: List of paths to open on startup, may be empty.
    """

    cu.get_log_path().parent.mkdir(parents=True, exist_ok=True)
    # Log up to 1MB to the log file.
    logzero.logfile(str(cu.get_log_path()), maxBytes=2**30, backupCount=1, loglevel=logzero.DEBUG)
    logger.info("\n---- Starting up ----")
    logger.info(f"Program: {__display_name__} {__version__}")
    logger.info(f"Log file is {cu.get_log_path()}")

    # # Set up icon theme.
    # if args.icon_theme:
    #     if args.icon_theme == "Breeze":
    #         logger.info("Using Breeze icon theme.")
    #         Qg.QIcon.setThemeName("Breeze")
    #     elif args.icon_theme == "BreezeDark":
    #         logger.info("Using BreezeDark icon theme.")
    #         Qg.QIcon.setThemeName("BreezeDark")
    #     else:
    #         raise ValueError(f"Unknown icon theme: {args.icon_theme}")
    # elif platform.system() == "Windows" or platform.system() == "Darwin":
    #     # Default to Breeze on Windows.
    #     logger.info("Using Breeze icon theme.")
    #     Qg.QIcon.setThemeName("Breeze")

    # Start the main window.
    app = Qw.QApplication(sys.argv)

    try:
        window = MainWindow()
        window.show()
        # TODO load the initial paths
        sys.exit(app.exec())
    except Exception as e:
        logger.exception(
            e,
        )
    finally:
        logger.info("---- Shutting down ----\n")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'Mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.4.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QCheckBox, QFormLayout,
    QFrame, QGroupBox, QHBoxLayout, QHeaderView,
    QLabel, QLineEdit, QMainWindow, QMenu,
    QMenuBar, QProgressBar, QPushButton, QRadioButton,
    QSizePolicy, QSpacerItem, QSplitter, QStatusBar,
    QTabWidget, QTableWidgetItem, QTextEdit, QVBoxLayout,
    QWidget)

from pcleaner.gui.CustomQ.CComboBox import CComboBox
from pcleaner.gui.file_table import FileTable

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1701, 720)
        self.action_open_files = QAction(MainWindow)
        self.action_open_files.setObjectName(u"action_open_files")
        icon = QIcon(QIcon.fromTheme(u"document-open"))
        self.action_open_files.setIcon(icon)
        self.action_clear_files = QAction(MainWindow)
        self.action_clear_files.setObjectName(u"action_clear_files")
        icon1 = QIcon(QIcon.fromTheme(u"edit-clear-history"))
        self.action_clear_files.setIcon(icon1)
        self.action_new_profile = QAction(MainWindow)
        self.action_new_profile.setObjectName(u"action_new_profile")
        icon2 = QIcon(QIcon.fromTheme(u"document-new"))
        self.action_new_profile.setIcon(icon2)
        self.action_delete_profile = QAction(MainWindow)
        self.action_delete_profile.setObjectName(u"action_delete_profile")
        icon3 = QIcon(QIcon.fromTheme(u"edit-delete"))
        self.action_delete_profile.setIcon(icon3)
        self.action_import_profile = QAction(MainWindow)
        self.action_import_profile.setObjectName(u"action_import_profile")
        icon4 = QIcon(QIcon.fromTheme(u"document-import"))
        self.action_import_profile.setIcon(icon4)
        self.action_open_log = QAction(MainWindow)
        self.action_open_log.setObjectName(u"action_open_log")
        self.action_panel_cleaner_documentation = QAction(MainWindow)
        self.action_panel_cleaner_documentation.setObjectName(u"action_panel_cleaner_documentation")
        self.action_source_code = QAction(MainWindow)
        self.action_source_code.setObjectName(u"action_source_code")
        self.actionView_License = QAction(MainWindow)
        self.actionView_License.setObjectName(u"actionView_License")
        self.action_save_profile = QAction(MainWindow)
        self.action_save_profile.setObjectName(u"action_save_profile")
        icon5 = QIcon(QIcon.fromTheme(u"document-save"))
        self.action_save_profile.setIcon(icon5)
        self.action_save_profile_as = QAction(MainWindow)
        self.action_save_profile_as.setObjectName(u"action_save_profile_as")
        icon6 = QIcon(QIcon.fromTheme(u"document-save-as"))
        self.action_save_profile_as.setIcon(icon6)
        self.action_Apply_Denoising = QAction(MainWindow)
        self.action_Apply_Denoising.setObjectName(u"action_Apply_Denoising")
        self.action_Apply_Denoising.setCheckable(True)
        self.action_Apply_Denoising.setChecked(True)
        self.action_show_terminal_command = QAction(MainWindow)
        self.action_show_terminal_command.setObjectName(u"action_show_terminal_command")
        self.action_system_theme = QAction(MainWindow)
        self.action_system_theme.setObjectName(u"action_system_theme")
        self.action_dark = QAction(MainWindow)
        self.action_dark.setObjectName(u"action_dark")
        self.action_light = QAction(MainWindow)
        self.action_light.setObjectName(u"action_light")
        self.action_temp = QAction(MainWindow)
        self.action_temp.setObjectName(u"action_temp")
        self.action_temp_2 = QAction(MainWindow)
        self.action_temp_2.setObjectName(u"action_temp_2")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(1, 4, 1, 1)
        self.splitter = QSplitter(self.centralwidget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setHandleWidth(10)
        self.groupBox = QGroupBox(self.splitter)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout_2 = QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.comboBox_current_profile = CComboBox(self.groupBox)
        self.comboBox_current_profile.addItem("")
        self.comboBox_current_profile.setObjectName(u"comboBox_current_profile")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox_current_profile.sizePolicy().hasHeightForWidth())
        self.comboBox_current_profile.setSizePolicy(sizePolicy)
        self.comboBox_current_profile.setMinimumSize(QSize(48, 0))

        self.horizontalLayout_2.addWidget(self.comboBox_current_profile)

        self.pushButton_save_profile = QPushButton(self.groupBox)
        self.pushButton_save_profile.setObjectName(u"pushButton_save_profile")
        self.pushButton_save_profile.setEnabled(False)
        icon7 = QIcon()
        iconThemeName = u"document-save"
        if QIcon.hasThemeIcon(iconThemeName):
            icon7 = QIcon.fromTheme(iconThemeName)
        else:
            icon7.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_save_profile.setIcon(icon7)

        self.horizontalLayout_2.addWidget(self.pushButton_save_profile)

        self.pushButton_reset_profile = QPushButton(self.groupBox)
        self.pushButton_reset_profile.setObjectName(u"pushButton_reset_profile")
        self.pushButton_reset_profile.setEnabled(False)
        icon8 = QIcon()
        iconThemeName = u"document-revert"
        if QIcon.hasThemeIcon(iconThemeName):
            icon8 = QIcon.fromTheme(iconThemeName)
        else:
            icon8.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_reset_profile.setIcon(icon8)

        self.horizontalLayout_2.addWidget(self.pushButton_reset_profile)

        self.pushButton_apply_profile = QPushButton(self.groupBox)
        self.pushButton_apply_profile.setObjectName(u"pushButton_apply_profile")
        self.pushButton_apply_profile.setEnabled(False)
        icon9 = QIcon()
        iconThemeName = u"dialog-ok-apply"
        if QIcon.hasThemeIcon(iconThemeName):
            icon9 = QIcon.fromTheme(iconThemeName)
        else:
            icon9.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_apply_profile.setIcon(icon9)

        self.horizontalLayout_2.addWidget(self.pushButton_apply_profile)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.toolBox_profile_frame = QWidget(self.groupBox)
        self.toolBox_profile_frame.setObjectName(u"toolBox_profile_frame")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.toolBox_profile_frame.sizePolicy().hasHeightForWidth())
        self.toolBox_profile_frame.setSizePolicy(sizePolicy1)

        self.verticalLayout_2.addWidget(self.toolBox_profile_frame)

        self.splitter.addWidget(self.groupBox)
        self.tabWidget_files = QTabWidget(self.splitter)
        self.tabWidget_files.setObjectName(u"tabWidget_files")
        self.tabWidget_table_page = QWidget()
        self.tabWidget_table_page.setObjectName(u"tabWidget_table_page")
        self.verticalLayout_3 = QVBoxLayout(self.tabWidget_table_page)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.file_table = FileTable(self.tabWidget_table_page)
        if (self.file_table.columnCount() < 5):
            self.file_table.setColumnCount(5)
        __qtablewidgetitem = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(2, __qtablewidgetitem2)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(3, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(4, __qtablewidgetitem4)
        self.file_table.setObjectName(u"file_table")
        self.file_table.setFrameShape(QFrame.NoFrame)
        self.file_table.setFrameShadow(QFrame.Plain)
        self.file_table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.file_table.setDragDropOverwriteMode(False)
        self.file_table.setDragDropMode(QAbstractItemView.NoDragDrop)
        self.file_table.setAlternatingRowColors(True)
        self.file_table.setSelectionMode(QAbstractItemView.SingleSelection)
        self.file_table.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.file_table.setSortingEnabled(True)
        self.file_table.setCornerButtonEnabled(False)

        self.verticalLayout_3.addWidget(self.file_table)

        self.tabWidget_files.addTab(self.tabWidget_table_page, "")
        self.splitter.addWidget(self.tabWidget_files)
        self.frame_3 = QFrame(self.splitter)
        self.frame_3.setObjectName(u"frame_3")
        self.frame_3.setFrameShape(QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QFrame.Raised)
        self.verticalLayout_4 = QVBoxLayout(self.frame_3)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.groupBox_4 = QGroupBox(self.frame_3)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.verticalLayout_9 = QVBoxLayout(self.groupBox_4)
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.radioButton_cleaning = QRadioButton(self.groupBox_4)
        self.radioButton_cleaning.setObjectName(u"radioButton_cleaning")
        self.radioButton_cleaning.setChecked(True)

        self.verticalLayout_9.addWidget(self.radioButton_cleaning)

        self.radioButton_ocr = QRadioButton(self.groupBox_4)
        self.radioButton_ocr.setObjectName(u"radioButton_ocr")

        self.verticalLayout_9.addWidget(self.radioButton_ocr)

        self.verticalSpacer = QSpacerItem(20, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_9.addItem(self.verticalSpacer)

        self.verticalLayout_7 = QVBoxLayout()
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.label_warning = QLabel(self.groupBox_4)
        self.label_warning.setObjectName(u"label_warning")
        sizePolicy2 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.label_warning.sizePolicy().hasHeightForWidth())
        self.label_warning.setSizePolicy(sizePolicy2)

        self.verticalLayout_7.addWidget(self.label_warning)

        self.pushButton_start = QPushButton(self.groupBox_4)
        self.pushButton_start.setObjectName(u"pushButton_start")
        icon10 = QIcon()
        iconThemeName = u"media-playback-start"
        if QIcon.hasThemeIcon(iconThemeName):
            icon10 = QIcon.fromTheme(iconThemeName)
        else:
            icon10.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_start.setIcon(icon10)

        self.verticalLayout_7.addWidget(self.pushButton_start)

        self.pushButton_abort = QPushButton(self.groupBox_4)
        self.pushButton_abort.setObjectName(u"pushButton_abort")
        icon11 = QIcon()
        iconThemeName = u"process-stop"
        if QIcon.hasThemeIcon(iconThemeName):
            icon11 = QIcon.fromTheme(iconThemeName)
        else:
            icon11.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_abort.setIcon(icon11)

        self.verticalLayout_7.addWidget(self.pushButton_abort)


        self.verticalLayout_9.addLayout(self.verticalLayout_7)


        self.horizontalLayout_3.addWidget(self.groupBox_4)

        self.groupBox_3 = QGroupBox(self.frame_3)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.verticalLayout_5 = QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.checkBox_save_clean = QCheckBox(self.groupBox_3)
        self.checkBox_save_clean.setObjectName(u"checkBox_save_clean")
        self.checkBox_save_clean.setChecked(True)

        self.verticalLayout_5.addWidget(self.checkBox_save_clean)

        self.checkBox_save_mask = QCheckBox(self.groupBox_3)
        self.checkBox_save_mask.setObjectName(u"checkBox_save_mask")
        self.checkBox_save_mask.setChecked(True)

        self.verticalLayout_5.addWidget(self.checkBox_save_mask)

        self.checkBox_save_text = QCheckBox(self.groupBox_3)
        self.checkBox_save_text.setObjectName(u"checkBox_save_text")

        self.verticalLayout_5.addWidget(self.checkBox_save_text)

        self.verticalSpacer_2 = QSpacerItem(20, 1, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_5.addItem(self.verticalSpacer_2)

        self.label_4 = QLabel(self.groupBox_3)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setTextInteractionFlags(Qt.LinksAccessibleByMouse)

        self.verticalLayout_5.addWidget(self.label_4)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.lineEdit_out_directory = QLineEdit(self.groupBox_3)
        self.lineEdit_out_directory.setObjectName(u"lineEdit_out_directory")

        self.horizontalLayout.addWidget(self.lineEdit_out_directory)

        self.pushButton_browse_out_dir = QPushButton(self.groupBox_3)
        self.pushButton_browse_out_dir.setObjectName(u"pushButton_browse_out_dir")
        icon12 = QIcon()
        iconThemeName = u"document-open-folder"
        if QIcon.hasThemeIcon(iconThemeName):
            icon12 = QIcon.fromTheme(iconThemeName)
        else:
            icon12.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)
        
        self.pushButton_browse_out_dir.setIcon(icon12)

        self.horizontalLayout.addWidget(self.pushButton_browse_out_dir)


        self.verticalLayout_5.addLayout(self.horizontalLayout)


        self.horizontalLayout_3.addWidget(self.groupBox_3)

        self.horizontalLayout_3.setStretch(0, 1)
        self.horizontalLayout_3.setStretch(1, 3)

        self.verticalLayout_4.addLayout(self.horizontalLayout_3)

        self.textEdit_analytics = QTextEdit(self.frame_3)
        self.textEdit_analytics.setObjectName(u"textEdit_analytics")
        self.textEdit_analytics.setUndoRedoEnabled(False)
        self.textEdit_analytics.setLineWrapMode(QTextEdit.NoWrap)
        self.textEdit_analytics.setLineWrapColumnOrWidth(100)
        self.textEdit_analytics.setReadOnly(True)

        self.verticalLayout_4.addWidget(self.textEdit_analytics)

        self.verticalLayout_4.setStretch(1, 1)
        self.splitter.addWidget(self.frame_3)

        self.verticalLayout.addWidget(self.splitter)

        self.widget_progress_drawer = QWidget(self.centralwidget)
        self.widget_progress_drawer.setObjectName(u"widget_progress_drawer")
        self.formLayout = QFormLayout(self.widget_progress_drawer)
        self.formLayout.setObjectName(u"formLayout")
        self.formLayout.setVerticalSpacing(4)
        self.label_progress_step = QLabel(self.widget_progress_drawer)
        self.label_progress_step.setObjectName(u"label_progress_step")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label_progress_step)

        self.progressBar_individual = QProgressBar(self.widget_progress_drawer)
        self.progressBar_individual.setObjectName(u"progressBar_individual")
        self.progressBar_individual.setValue(24)

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.progressBar_individual)

        self.label_2 = QLabel(self.widget_progress_drawer)
        self.label_2.setObjectName(u"label_2")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.label_2)

        self.progressBar_total = QProgressBar(self.widget_progress_drawer)
        self.progressBar_total.setObjectName(u"progressBar_total")
        self.progressBar_total.setValue(24)
        self.progressBar_total.setTextVisible(True)

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.progressBar_total)


        self.verticalLayout.addWidget(self.widget_progress_drawer)

        self.verticalLayout.setStretch(0, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1701, 34))
        self.menu_File = QMenu(self.menubar)
        self.menu_File.setObjectName(u"menu_File")
        self.menu_Profile = QMenu(self.menubar)
        self.menu_Profile.setObjectName(u"menu_Profile")
        self.menu_set_default_profile = QMenu(self.menu_Profile)
        self.menu_set_default_profile.setObjectName(u"menu_set_default_profile")
        self.menu_settings = QMenu(self.menubar)
        self.menu_settings.setObjectName(u"menu_settings")
        self.menu_theme = QMenu(self.menu_settings)
        self.menu_theme.setObjectName(u"menu_theme")
        self.menu_Help = QMenu(self.menubar)
        self.menu_Help.setObjectName(u"menu_Help")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
#if QT_CONFIG(shortcut)
        self.label_4.setBuddy(self.lineEdit_out_directory)
#endif // QT_CONFIG(shortcut)

        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Profile.menuAction())
        self.menubar.addAction(self.menu_settings.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.menu_File.addAction(self.action_open_files)
        self.menu_File.addAction(self.action_clear_files)
        self.menu_Profile.addAction(self.action_new_profile)
        self.menu_Profile.addAction(self.action_import_profile)
        self.menu_Profile.addSeparator()
        self.menu_Profile.addAction(self.action_save_profile)
        self.menu_Profile.addAction(self.action_save_profile_as)
        self.menu_Profile.addSeparator()
        self.menu_Profile.addAction(self.action_delete_profile)
        self.menu_Profile.addSeparator()
        self.menu_Profile.addAction(self.menu_set_default_profile.menuAction())
        self.menu_set_default_profile.addAction(self.action_temp_2)
        self.menu_settings.addAction(self.menu_theme.menuAction())
        self.menu_settings.addAction(self.action_open_log)
        self.menu_theme.addAction(self.action_system_theme)
        self.menu_theme.addSeparator()
        self.menu_theme.addAction(self.action_dark)
        self.menu_theme.addAction(self.action_light)
        self.menu_Help.addAction(self.action_panel_cleaner_documentation)
        self.menu_Help.addAction(self.action_source_code)
        self.menu_Help.addSeparator()
        self.menu_Help.addAction(self.action_show_terminal_command)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Panel Cleaner", None))
        self.action_open_files.setText(QCoreApplication.translate("MainWindow", u"Open Files", None))
        self.action_clear_files.setText(QCoreApplication.translate("MainWindow", u"Clear Files", None))
        self.action_new_profile.setText(QCoreApplication.translate("MainWindow", u"New", None))
        self.action_delete_profile.setText(QCoreApplication.translate("MainWindow", u"Delete", None))
        self.action_import_profile.setText(QCoreApplication.translate("MainWindow", u"Import...", None))
        self.action_open_log.setText(QCoreApplication.translate("MainWindow", u"Open Log", None))
        self.action_panel_cleaner_documentation.setText(QCoreApplication.translate("MainWindow", u"Online Documentation", None))
        self.action_source_code.setText(QCoreApplication.translate("MainWindow", u"About Panel Cleaner", None))
        self.actionView_License.setText(QCoreApplication.translate("MainWindow", u"View License", None))
        self.action_save_profile.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.action_save_profile_as.setText(QCoreApplication.translate("MainWindow", u"Save as...", None))
        self.action_Apply_Denoising.setText(QCoreApplication.translate("MainWindow", u"Apply Denoising", None))
        self.action_show_terminal_command.setText(QCoreApplication.translate("MainWindow", u"Show Terminal Command", None))
        self.action_system_theme.setText(QCoreApplication.translate("MainWindow", u"System", None))
        self.action_dark.setText(QCoreApplication.translate("MainWindow", u"Dark", None))
        self.action_light.setText(QCoreApplication.translate("MainWindow", u"Light", None))
        self.action_temp.setText(QCoreApplication.translate("MainWindow", u"<temp>", None))
        self.action_temp_2.setText(QCoreApplication.translate("MainWindow", u"<temp>", None))
        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"Profile", None))
        self.comboBox_current_profile.setItemText(0, QCoreApplication.translate("MainWindow", u"Default", None))

        self.pushButton_save_profile.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.pushButton_reset_profile.setText(QCoreApplication.translate("MainWindow", u"Reset All", None))
        self.pushButton_apply_profile.setText(QCoreApplication.translate("MainWindow", u"Apply", None))
        ___qtablewidgetitem = self.file_table.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("MainWindow", u"id", None));
        ___qtablewidgetitem1 = self.file_table.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("MainWindow", u"File", None));
        ___qtablewidgetitem2 = self.file_table.horizontalHeaderItem(2)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("MainWindow", u"Size", None));
        ___qtablewidgetitem3 = self.file_table.horizontalHeaderItem(3)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("MainWindow", u"Status", None));
        ___qtablewidgetitem4 = self.file_table.horizontalHeaderItem(4)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("MainWindow", u"Output", None));
        self.tabWidget_files.setTabText(self.tabWidget_files.indexOf(self.tabWidget_table_page), QCoreApplication.translate("MainWindow", u"Images", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("MainWindow", u"Process", None))
        self.radioButton_cleaning.setText(QCoreApplication.translate("MainWindow", u"Cleaning", None))
        self.radioButton_ocr.setText(QCoreApplication.translate("MainWindow", u"OCR", None))
        self.label_warning.setText(QCoreApplication.translate("MainWindow", u"Warning", None))
        self.pushButton_start.setText(QCoreApplication.translate("MainWindow", u"Start", None))
        self.pushButton_abort.setText(QCoreApplication.translate("MainWindow", u"Abort", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("MainWindow", u"Output", None))
        self.checkBox_save_clean.setText(QCoreApplication.translate("MainWindow", u"Cleaned Image", None))
        self.checkBox_save_mask.setText(QCoreApplication.translate("MainWindow", u"Mask", None))
        self.checkBox_save_text.setText(QCoreApplication.translate("MainWindow", u"Extracted Text", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Output Directory:", None))
        self.pushButton_browse_out_dir.setText("")
        self.textEdit_analytics.setHtml(QCoreApplication.translate("MainWindow", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Noto Mono','Monospace';\">Mask Fitment Analytics <br />---------------------- <br />Total boxes: 5 | Masks succeeded: 5 (100%) | Masks failed: </span><span style=\" font-family:'Noto Mono','Monospace'; color:#b21818;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> <br />Perfect masks: </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">5</span><span style=\" font-family:'Noto Mono','Monospace';\"> (100%) | Average border deviation: 0.00 <br /><br />Mask usage by mask size (smallest to largest)"
                        ": <br />Mask 0  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 1  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 2  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 3  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 4  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 5  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 6  : </span><span style=\" font-family:'Noto Mono','"
                        "Monospace'; color:#18b2b2;\">\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588</span><span style=\" font-family:'Noto Mono','Monospace';\"> </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">1</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 1 <br />Mask 7  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 8  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 9  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 10 :  </span><span style=\" font-family:'Noto Mono','Monospa"
                        "ce'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Box mask: </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588</span><span style=\" font-family:'Noto Mono','Monospace';\"> </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">4</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 4 <br /><br /></span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">\u2588 Perfect</span><span style=\" font-family:'Noto Mono','Monospace';\"> | \u2588 Total<br /><br /></span></p></body></html>", None))
        self.label_progress_step.setText(QCoreApplication.translate("MainWindow", u"Current Step:", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Total Progress:", None))
        self.menu_File.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menu_Profile.setTitle(QCoreApplication.translate("MainWindow", u"Profile", None))
        self.menu_set_default_profile.setTitle(QCoreApplication.translate("MainWindow", u"Set Default", None))
        self.menu_settings.setTitle(QCoreApplication.translate("MainWindow", u"Settings", None))
        self.menu_theme.setTitle(QCoreApplication.translate("MainWindow", u"Theme", None))
        self.menu_Help.setTitle(QCoreApplication.translate("MainWindow", u"Help", None))
    # retranslateUi

